<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2207bf7dd286 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-02 13:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class liveUpdateResponse{
	/** @var string */
	var $Type; //type of response

	/** @var string */
	var $Headline; //for template-type

	/** @var string */
	var $Content; //for template-type

	/** @var string */
	var $Header; //for template-type

	/** @var string */
	public $Message;

	/** @var string */
	public $SuccessText;

	/** @var array */
	public $Repeat;

	/** @var string */
	var $Code; //for eval-type

	/** @var array<string,string|array> */
	var $Error = [];

	/** @var string */
	var $State = '';

	/** @var string */
	var $LogMsg = '';

	/** @var array<string,string> */
	var $Files = [];

	/** @var array<string,int|string> */
	var $Parts = [];

	/** @var string */
	var $EncodedCode = '';

	/** @var bool */
	var $Encoding = false;

	/** @var array<string,string,array> */
	public $Next = [];

	/** @var int */
	public $From = 0;

	/** @var int */
	public $To = 0;

	/** @var array <string,string|array> */
	public $Lang = [];

	/** @var string */
	public $ClientVersion = '0';

	/** @var string */
	protected $License = '';

	/** @var string */
	protected $TargetVersion = '';

	/** @var array<string,mixed> */
	protected $Replace = [];

	protected function getErrorMsg(): string{
		$msg = '<div class="errorDiv">' .
			$this->Error['headline'] . '<br />' .
			$this->Error['message'] .
			($GLOBALS['liveUpdateError']['errorString'] ? $this->Error['lang']['errorMessage'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorString'] . '</code><br />' .
			$this->Error['lang']['errorIn'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorFile'] . '</code><br />' .
			$this->Error['lang']['errorLine'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorLine'] . '</code>' : '') .
			'</div>';

		return '<script><!--
	top.frames.updatecontent.appendMessageLog("' . strtr($msg, ['"' => "'", "\n" => ' ']) . '");
	alert("' . strtr(strip_tags($msg), ['"' => "'", "\n" => ' ']) . '");
//-->
</script>';
	}

	protected function getProgress(string $msg, array $urls): string{
		return '<script>
			top.frames.updatecontent.decreaseSpeed = false;
			top.frames.updatecontent.nextUrl = "' . $urls['nextUrl'] . '"+(top.frames.updatecontent.param?top.frames.updatecontent.param:"");
			top.frames.updatecontent.setProgressBar("' . $urls['progress'] . '");
var msg="' . strtr($msg . $urls['message'], ["\r" => '', "\n" => '', '"' => "'"]) . '\n";
			top.frames.updatecontent.appendMessageLog(msg);' .
			($urls['nextStep'] ? 'top.frames.updatecontent.finishLiInstallerStep("' . $urls['nextStep'][0] . '");
			top.frames.updatecontent.activateLiInstallerStep("' . $urls['nextStep'][1] . '");' : '') . '
			window.setTimeout("top.frames.updatecontent.proceedUrl();", 20);
		</script>';
	}

	protected function finishUpdateJS(string $msg): string{
		return'<script>
top.frames["updatecontent"].setProgressBar(100);
top.frames["updatecontent"].appendMessageLog("' . $msg . '\n");
			alert("' . strip_tags($msg) . '");
top.opener.top.we_cmd("dologout");
		</script>';
	}

	function initByArray(array $respArray): void{
		foreach($respArray as $key => $value){
			$this->{$key} = $value;
		}

		if($this->Encoding && $this->EncodedCode){
			$this->Code = base64_decode($this->EncodedCode) ?: '';
		}
	}

	/**
	 * init the object with the response from the update-server
	 *
	 * @param string $response
	 * @return boolean
	 */
	function initByHttpResponse(string $response): bool{
		if(($respArr = liveUpdateResponse::responseToArray($response))){
			$this->initByArray($respArr);
			return true;
		}
		return false;
	}

	function isError(): bool{
		return ($this->Type === 'state' && $this->State === 'error');
	}

	function getField(string $fieldname): string{
		return $this->{$fieldname} ?? '';
	}

	protected function checkfreequota(int &$AnzMB): bool{
		$kB = str_repeat('0', 1024);
		$MB = str_repeat($kB, 1024);

		if(!is_dir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp')){
			mkdir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp');
		}
		$testfilename = LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp/testquota.txt';

		if(file_exists($testfilename)){
			unlink($testfilename);
		}
		$allesOK = true;
		for($i = 1; $i <= $AnzMB; $i++){
			if(!file_put_contents($testfilename, $MB, FILE_APPEND)){
				$allesOK = false;
				break;
			}
		}
		$AnzMB = $i - 1;
		unlink($testfilename);
		return $allesOK;
	}

	protected function changesResponse(): string{
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);

		$testdiskquota = 100;
		$diskquotawarning = $this->checkfreequota($testdiskquota);

		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);
		if(!$diskquotawarning){
			return strtr($this->getErrorMsg(), ['__QUOTA__' => $testdiskquota]);
		}

		if(method_exists($liveUpdateFnc, 'writeLicenseToSession')){
			$liveUpdateFnc->writeLicenseToSession($this->License);
		}
		$liveUpdateFnc->insertUpdateLogEntry($this->LogMsg, $this->TargetVersion, 0);
		return $this->getProgress('', $this->Next);
	}

	protected function executePatches(): string{
		global $liveUpdateFnc;
		if(method_exists($liveUpdateFnc, 'writeLicenseToDefines')){
			$liveUpdateFnc->writeLicenseToDefines();
		}
		$redo = liveUpdateFunctionsServer::weUpdaterDoUpdate(we_base_request::string('progress', '', 'what'), (empty($_REQUEST['progress']) ? [] : (array) $_REQUEST['progress']));
		$allFiles = $liveUpdateFnc->getFilesDir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/patches/');

		$errorFile = '';

		foreach($allFiles as $file){
			//$message .= basename($file) . '<br />';
			$success = $liveUpdateFnc->executePatch($file);
			if(!$success){
				$errorFile .= ' ' . basename($file);
			}
			unlink($file);
		}

		if($errorFile){
			return strtr($this->getErrorMsg(), [
				'__HEAD__' => $errorFile,
				'__MSG__' => $GLOBALS["errorDetail"]
			]);
			}
		$message = ($allFiles ? '<div>' . $this->Message . '</div>' : '');

		if(!empty($redo)){
				$message .= '<div>Update ' . $redo['text'] . '</div>';
				unset($redo['text']);
				return '<script>top.frames.updatecontent.param="&' . http_build_query(['progress' => $redo]) . '";</script>' .
					$this->getProgress($message, $this->Repeat);
			}

			$delDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/includes/';
			if(is_file($delDir . "del.files") && method_exists($liveUpdateFnc, 'removeObsoleteFiles')){
				if(is_file($delDir . 'deleted.files')){
					unlink($delDir . 'deleted.files');
				}
				$liveUpdateFnc->removeObsoleteFiles($delDir);
			}
			if(method_exists($liveUpdateFnc, 'removeDirOnlineInstaller')){
				$liveUpdateFnc->removeDirOnlineInstaller();
			}

			return $this->getProgress($message, $this->Next);
		}

	function responseToArray(string $response): array{
		$data = base64_decode($response) ?: '';
		$respArray = $data ? we_unserialize($data) : [];
		return (is_array($respArray) ? $respArray : []);
	}

	protected function saveFiles(): string{
		global $liveUpdateFnc;

		$mExist = method_exists($liveUpdateFnc, 'checkMakeFileWritable');
		foreach($this->Files as $path => $content){
			$content = base64_decode($content);
			$tmpPath = ltrim($path, '/');
			$testPath = strpos($tmpPath, 'tmp/files') === 0 ? '/' . ltrim(str_replace('tmp/files', '', $tmpPath), '/') : false;

			if($liveUpdateFnc->isPhpFile($path)){
				$content = $liveUpdateFnc->preparePhpCode($content);
			}

			if(!$liveUpdateFnc->filePutContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $path, $content)){
				$this->Error = str_replace('__PATH__', $path, $this->Error['path']);
				return $this->getErrorMsg();
			}

			if($testPath && $mExist && !$liveUpdateFnc->checkMakeFileWritable($testPath)){
				$this->Error = str_replace('__PATH__', $testPath, $this->Error['write']);
				return $this->getErrorMsg();
			}
			//$successFiles[] = $path;
		}

		if($this->Parts){//multipart files to merge
			$content = '';
			for($i = 0; $i <= $this->Parts['Count']; $i++){
				$content .= $liveUpdateFnc->getFileContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'] . 'part' . $i);
				$liveUpdateFnc->deleteFile(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'] . 'part' . $i);
			}

			if($liveUpdateFnc->isPhpFile($this->Parts['Name'])){
				$content = $liveUpdateFnc->preparePhpCode($content);
			}

			if($liveUpdateFnc->filePutContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'], $content)){
				//$successFiles[] = $this->Parts['Name'];
			}
		}

		$message = $this->SuccessText . '<br/>';
		return $this->getProgress($message, $this->Next);
	}

	protected function dbUpdate(): string{
		global $liveUpdateFnc;
		$queryDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/queries/';

		$allFiles = $liveUpdateFnc->getFilesDir($queryDir);
		sort($allFiles);

		$message = '';

		for($i = $this->From; $i < count($allFiles) && $i < $this->To; $i++){
			// execute queries in each file
			$fileName = basename($allFiles[$i]);
			if($liveUpdateFnc->executeQueriesInFiles($allFiles[$i])){
				$text = substr($fileName, -40);
				$message .= "<div>&hellip;$text</div>";
			}
			$msg = array_filter($liveUpdateFnc->getQueryLog());
			unset($msg['success']);
			foreach($msg as $key => $vals){
				$err = ($key === 'error' ? 1 : 2);
				$message .= "<div class='" . ($err === 1 ? 'errorDiv' : 'messageDiv') . "'>" . $this->Lang['notice'] . '<br />' . $fileName . ': ' . $this->Lang[$key] . '</div>';
				$liveUpdateFnc->insertQueryLogEntries($key, $fileName . ': ' . $this->Lang[$key] . implode(',', $vals), $err, $this->ClientVersion);
			}
			$liveUpdateFnc->clearQueryLog();
		}
		if(count($allFiles) > $this->To){ // continue with DB steps
			return $this->getProgress($message, $this->Repeat);
		}// proceed to next step.
		return $this->getProgress($message, $this->Next);
	}

	protected function copyFiles(): string{
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/files/';
		$preLength = strlen($filesDir);

		$allFiles = $liveUpdateFnc->getFilesDir($filesDir);

		foreach($allFiles as $file){
			$success = $liveUpdateFnc->moveFile($file, LIVEUPDATE_SOFTWARE_DIR . substr($file, $preLength));
			if(!$success){
				return $this->getErrorMsg();
			}
		}

		$message = '<div>' . $this->Message . '</div>';

		return $this->getProgress($message, $this->Next);
	}

	protected function finishLangUpdate(): string{
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);
		$liveUpdateFnc->insertUpdateLogEntry($this->LogMsg, $this->TargetVersion, 0);
		$message = '<div>' . $this->Message . '</div>';
		return $this->finishUpdateJS($message);
	}

	protected function finishUpdate(): string{
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);
		$success = false;
		if($liveUpdateFnc->replaceCode(LIVEUPDATE_SOFTWARE_DIR . $this->Replace['path'], $this->Replace['replace'], $this->Replace['needle'])){
			$liveUpdateFnc->insertUpdateLogEntry($this->LogMsg, $this->TargetVersion, 0);
			$success = true;
		}
		$liveUpdateFnc->deleteDir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/updateClient');
		//no real error message defined....
		return $success ? $this->finishUpdateJS('<div>' . $this->Message . '</div>') : $this->getErrorMsg();
	}

	function getOutput(): string{
		if(!class_exists('liveUpdateFunctionsServer', false)){
			class_alias('liveUpdateFunctions', 'liveUpdateFunctionsServer');
		}

		if(empty($GLOBALS['liveUpdateFnc'])){
			$GLOBALS['liveUpdateFnc'] = new liveUpdateFunctionsServer();
			set_error_handler(['liveUpdateFunctionsServer', 'liveUpdateErrorHandler']);
		}

		switch($this->Type){
			case 'template':
				return welu_update_templates::getHtml($this->Headline, $this->Content, $this->Header ?? '');
			case 'ChangesResponse':
				return $this->changesResponse();
			case 'SaveFiles':
				return $this->saveFiles();
			case 'DBUpdate':
				return $this->dbUpdate();
			case 'CopyFiles':
				return $this->copyFiles();
			case 'ExecutePatches':
				return $this->executePatches();
			case 'nextStep':
				return '';
			case 'finishLangUpdate':
				return $this->finishLangUpdate();
			case 'finishUpdate':
				return $this->finishUpdate();
			case 'state':
				//FIXME: is this used?!
				return welu_update_frames::htmlStateMessage('');
			default:
				t_e($this->Type . ' is not implemented yet', $this);
				return $this->Type . ' is not implemented yet';
		}
	}

}
