<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 11384bf894dc $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-10 19:00 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 *  view class for document customer filters
 *
 */
class we_customer_documentFilterView extends we_customer_filterView{
	/** @var we_customer_documentFilter */
	protected $filter = null;

	public function __construct(we_customer_abstractFilter &$filter, int $width = 0){
		parent::__construct($filter, $width);
		$this->viewType = 'document';
	}

	/**
	 * Gets the HTML and Javascript for the filter
	 *
	 * @return string
	 */
	function getFilterHTML(bool $ShowModeNone = false): string{
		return parent::getFilterHTML() .
				(new we_html_multiIconBox($this->getAccessControlHTML()
					->append((in_array($this->we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES) ? $this->getFolderApplyHTML() : new we_html_multiIconBoxContent()))))
				->get();
	}

	/**
	 * Gets the HTML and Javascript for the access control ui
	 *
	 * @return we_html_multiIconBoxContent
	 */
	private function getAccessControlHTML(): we_html_multiIconBoxContent{
		$filter = $this->filter;

		$weSuggest = we_gui_suggest::getInstance();

		/*		 * ** AUTOSELECTOR FOR ErrorDocument, Customer is not logged in *** */
		$id_selectorNoLoginId = $filter->getErrorDocNoLogin();
		$path_selectorNoLoginId = $id_selectorNoLoginId ? id2path($id_selectorNoLoginId) : "";
		if(!$path_selectorNoLoginId){
			$id_selectorNoLoginId = 0;
		}

		$selectorNoLoginId = 'wecf_noLoginId';
		$selectorNoLoginText = 'wecf_InputNoLoginText';
		//$selectorNoLoginError = "wecf_ErrorMarkNoLoginText";

		$weSuggest->setAcId('NoLogin')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT])
			->setInput($selectorNoLoginText, $path_selectorNoLoginId)
			->setLabel(g_l('modules_customerFilter', '[documentNoLogin]'))
			->setMaxResults(20)
			->setResult($selectorNoLoginId, strval($id_selectorNoLoginId))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(409)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $selectorNoLoginId . "'].value,table:WE().consts.tables.FILE_TABLE,targetID:'" . $selectorNoLoginId . "',targetName:'" . $selectorNoLoginText . "',jsCmd:'setHot',filter:WE().consts.contentTypes.WEDOCUMENT,open_doc:true})") . "<div id=\"wecf_container_noLoginId\"></div>");

		$weAcSelector = $weSuggest->getHTML();

		/*		 * ** AUTOSELECTOR FOR ErrorDocument, Customer might be logged in, but has no access *** */
		$id_selectorNoAccessId = $filter->getErrorDocNoAccess();
		$path_selectorNoAccessId = $id_selectorNoAccessId ? id2path($id_selectorNoAccessId) : "";
		if(!$path_selectorNoAccessId){
			$id_selectorNoAccessId = '';
		}

		$selectorNoAccessId = 'wecf_noAccessId';
		$selectorNoAccessText = 'wecf_InputNoAccessText';
		//$selectorNoAccessError = "wecf_ErrorMarkNoAccessText";

		$weSuggest->setAcId('NoAccess')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT])
			->setInput($selectorNoAccessText, $path_selectorNoAccessId)
			->setLabel(g_l('modules_customerFilter', '[documentNoAccess]'))
			->setMaxResults(20)
			->setResult($selectorNoAccessId, strval($id_selectorNoAccessId))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(409)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $selectorNoAccessId . "'].value,table:WE().consts.tables.FILE_TABLE,targetID:'" . $selectorNoAccessId . "',targetName:'" . $selectorNoAccessText . "',jsCmd:'setHot',filter:WE().consts.contentTypes.WEDOCUMENT,open_doc:true})"));

		$weAcSelector2 = $weSuggest->getHTML();

		$accesControl = (new we_html_multiIconBox((new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('modules_customerFilter', '[accessControl]'), we_html_forms::radiobutton(
							"onTemplate", $filter->getAccessControlOnTemplate(), "wecf_accessControlOnTemplate", g_l('modules_customerFilter', '[accessControlOnTemplate]'), true, "", "updateView_document();top.content.setHot();") .
						we_html_forms::radiobutton(
							"errorDoc", !$filter->getAccessControlOnTemplate(), "wecf_accessControlOnTemplate", g_l('modules_customerFilter', '[accessControlOnErrorDoc]'), true, "", "updateView_document();top.content.setHot();") .
						we_customer_documentFilterView::getDiv($weAcSelector . $weAcSelector2, 'accessControlSelectorDiv', (!$filter->getAccessControlOnTemplate()), 25)))->noline()
				)))
			->get();


		return (new we_html_multiIconBoxContent())
				->add(
					(new we_html_multiIconBoxContentLine('',
																					we_html_element::div([
							'id' => 'accessControlDiv',
							'style' => 'display:' . ($filter->getMode() === we_customer_abstractFilter::OFF ? 'none' : 'block' ) . ';'
							], $accesControl)))
					->noline()
		);
	}

	/**
	 * Gets the HTML and Javascript for the folder apply ui (copy filter)
	 *
	 * @return we_html_multiIconBoxContent
	 */
	private function getFolderApplyHTML(): we_html_multiIconBoxContent{
		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(
						g_l('modules_customerFilter', '[apply_filter]') . we_html_tools::alertAttentionBox(g_l('modules_customerFilter', '[apply_filter_info]'), we_html_tools::TYPE_HELP, 0, false),
																																														 we_html_button::create_button(we_html_button::OK, "javascript:if (_EditorFrame.getEditorIsHot()) {WE().util.showMessage(WE().consts.g_l.alert.apply_filter_isHot,WE().consts.message.WE_MESSAGE_ERROR, window);} else { we_cmd('we_customer_copyWeDocumentFilter', '" . $this->we_doc->ID . "', '" . $this->we_doc->Table . "');}")))
					->noline()
					->rightHead()
		);
	}

}
