<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 06f4047033e8 $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-03 18:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_doclist_resultObjectfiles extends we_search_resultBase{
	/** @var int */
	public $ID = 0;

	/** @var int */
	protected static $tableID = 0;

	/** @var array */
	protected static $defaultValues = [];

	/** @var array<string, string|int|float|bool|DateTime> */
	public $fields = [];

	/** @var array<int,array{array{sort:string,title:string,field:string,type:string}}> */
	protected static $fieldColumns = [];

	public function __construct(we_search_search $searchclass){
		parent::__construct($searchclass);
		$this->type = we_doclist_view::DOCLIST_OBJECTFILES;
		$this->docTable = OBJECT_FILES_TABLE;
		$this->ContentType = we_base_ContentTypes::OBJECT_FILE;
	}

	/**
	 *
	 * @param int $id
	 */
	public function setID(int $id): self{
		$this->ID = $id;
		$this->docID = $id;
		return $this;
	}

	/**
	 *
	 * @param string $name
	 * @param int $value
	 */
	public function setFieldI(string $name, int $value): self{
		$this->fields[$name] = $value;

		return $this;
	}

	/**
	 *
	 * @param string $name
	 * @param float $value
	 */
	public function setFieldF(string $name, float $value): self{
		$this->fields[$name] = $value;

		return $this;
	}

	/**
	 *
	 * @param string $name
	 * @param string $value
	 */
	public function setFieldS(string $name, string $value): self{
		$this->fields[$name] = $value;

		return $this;
	}

	/**
	 *
	 * @param string $name
	 * @param bool $value
	 */
	public function setFieldB(string $name, bool $value): self{
		$this->fields[$name] = $value;

		return $this;
	}

	/**
	 *
	 * @param string $name
	 * @param DateTime $value
	 */
	public function setFieldD(string $name, DateTime $value): self{
		$this->fields[$name] = $value;

		return $this;
	}

	/**
	 * @return array<int,array{sort:string,title:string}>
	 */
	public static function getListColumns(): array{
		return [
			['sort' => '', 'title' => ''],
			['sort' => '', 'title' => ''],
			['sort' => '', 'title' => g_l('modules_objectClassfoldersearch', '[zeige]')],
			['sort' => '', 'title' => ''],
			['sort' => 'Path', 'title' => g_l('modules_objectClassfoldersearch', '[Objekt]')],
			['sort' => 'ID', 'title' => g_l('modules_objectClassfoldersearch', '[ID]')],
			['sort' => '', 'title' => g_l('modules_objectClassfoldersearch', '[Arbeitsbereiche]')],
			['sort' => 'Publish_Date', 'title' => g_l('modules_objectClassfoldersearch', '[Veroeffentlicht]')],
			['sort' => 'Mod_Date', 'title' => g_l('modules_objectClassfoldersearch', '[geaendert]')],
			['sort' => 'Url', 'title' => g_l('modules_objectClassfoldersearch', '[url]')],
			['sort' => 'TriggerID', 'title' => g_l('modules_objectClassfoldersearch', '[triggerid]')],
			['sort' => '', 'title' => g_l('modules_objectClassfoldersearch', '[language]')],
			['sort' => 'WebUserID', 'title' => g_l('modules_objectClassfoldersearch', '[WebUser]')],
		];
	}

	/**
	 *
	 * @param int $tableID
	 * @return array<int,array{sort:string,title:string,type:string,name?:string}>
	 */
	public static function getFieldColumns(int $tableID): array{
		if(empty(self::$fieldColumns[$tableID])){
			self::$fieldColumns[$tableID] = [
				['sort' => '', 'title' => '', 'type' => 'column_checkbox'],
				['sort' => '', 'title' => '', 'type' => 'column_icon'],
				['sort' => '', 'title' => g_l('modules_objectClassfoldersearch', '[zeige]'), 'type' => 'column_show'],
				['sort' => '', 'title' => '', 'type' => 'column_searchable'],
				['sort' => 'Path', 'title' => g_l('modules_objectClassfoldersearch', '[Objekt]'), 'type' => 'column_path'],
				['sort' => 'ID', 'title' => g_l('modules_objectClassfoldersearch', '[ID]'), 'type' => 'column_id'],
			];

			foreach(DB_WE::$inst->metaFieldNames(OBJECT_X_TABLE . $tableID) as $field){
				$parts = explode('_', $field, 2);
				switch($parts[0]){
					case 'OF':
					case 'variant':
						continue 2;
					case 'object':
						$title = f('SELECT Filename FROM ' . OBJECT_TABLE . ' WHERE ID=' . (int) $parts[1]);
						self::$fieldColumns[$tableID][] = ['sort' => '', 'title' => $title, 'name' => $field, 'type' => 'object'];
						break;
					default:
						self::$fieldColumns[$tableID][] = ['sort' => $field, 'title' => $parts[1], 'name' => $field, 'type' => $parts[0]];
				}
			}
		}

		return self::$fieldColumns[$tableID];
	}

	public function processHtml(): void{
		$this->processElementList();
		$this->processElementFields();
		self::$counter++;
	}

	protected function processElementList(): void{
		$htmlCheckbox = $this->getHtmlCheckbox();
		$htmlIcon = $this->getHtmlIcon();
		$htmlShow = $this->getHtmlShow();
		$htmlIsSearchable = $this->getHtmlSearchable();
		$htmlFilename = $this->getHtmlFilename();

		$htmlWorkspaces = $this->getHtmlWorkspaces(array_filter(explode(',', $this->Workspaces)));
		$htmlPublishDate = $this->Publish_Date ? (new DateTime($this->Publish_Date))->format(g_l('date', '[format][default]')) : '-';
		$htmlModDate = $this->Mod_Date ? (new DateTime($this->Mod_Date))->format(g_l('date', '[format][default]')) : '-';
		$htmlTriggerId = $this->TriggerID ? id2path($this->TriggerID) : '';

		$class = 'middlefont textCol';
		$this->elementList = (new we_html_tr([
				new we_html_td($htmlCheckbox, ['class' => $class . ' checkbox']),
				new we_html_td($htmlIcon, ['class' => $class . ' icon']),
				new we_html_td($htmlShow, ['class' => $class . ' show']),
				new we_html_td($htmlIsSearchable, ['class' => $class . ' searchable']),
				new we_html_td($htmlFilename, ['class' => $class . ' filename']),
				new we_html_td((string) $this->ID, ['class' => $class . ' id']),
				new we_html_td($htmlWorkspaces, ['class' => $class . ' workspaces']),
				new we_html_td($htmlPublishDate, ['class' => $class . ' publishdate']),
				new we_html_td($htmlModDate, ['class' => $class . ' moddate']),
				new we_html_td($this->Url, ['class' => $class . ' seourl']),
				new we_html_td($htmlTriggerId, ['class' => $class . ' triggerid']),
				new we_html_td($this->Language, ['class' => $class . ' language']),
				new we_html_td((string) $this->WebUserID, ['class' => $class . ' webuserid']),
				]))->getHtml();
	}

	protected function processElementFields(): void{
		$cols = [];
		foreach(self::getFieldColumns($this->TableID) as $col){
			'@phan-var array{sort:string,title:string,type:string,name?:string} $col';
			$colHtml = '';
			try{
				switch($col['type']){
					case 'column_checkbox':
						$colHtml = $this->getHtmlCheckbox();
						break;
					case 'column_icon':
						$colHtml = $this->getHtmlIcon();
						break;
					case 'column_show':
						$colHtml = $this->getHtmlShow();
						break;
					case 'column_searchable':
						$colHtml = $this->getHtmlSearchable();
						break;
					case 'column_path':
						$colHtml = $this->getHtmlFilename();
						break;
					case 'column_id':
						$colHtml = (string) $this->ID;
						break;
					case 'int':
					case 'country':
					case 'language':
					case 'shopCategory':
						$colHtml = (string) $this->fields[$col['name']];
						break;
					case 'img':
					case 'audio':
					case 'video':
					case 'flashmovie':
					case 'binary':
						$colHtml = $this->getHtmlBinary((int) $this->fields[$col['name']]);
						break;
					case 'collection':
						$colHtml = $this->getHtmlCollection((int) $this->fields[$col['name']]);
						break;
					case 'link':
						$colHtml = $this->getHtmlLink((string) $this->fields[$col['name']]);
						break;
					case 'href':
						$colHtml = we_document_base::getHrefByArray(we_unserialize((string) $this->fields[$col['name']]) ?: []);
						break;
					case 'object':
						if(($obj = (new DB_WE())->getHash('SELECT Path, IsPublished, IsModified FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($this->fields[$col['name']]) . ' AND IsFolder=0'))){
							$colHtml = '<a href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE,' . $this->fields[$col['name']] . ',\'' . we_base_ContentTypes::OBJECT_FILE . '\');" ' . ($obj['IsModified'] ? 'style="color:blue;"' : ($obj['IsPublished'] ? '' : 'style="color:blue;"')) . '" class="defaultfont" title="' . $obj['Path'] . '">' . we_base_util::shortenPath($obj['Path'], 32) . '</a>';
						}
						break;
					case we_contents_objectBase::TYPE_MULTIOBJECT:
						$temp = we_unserialize((string) $this->fields[$col['name']]);
						$objects = we_base_request::filterIntList($temp['objects'] ?? $temp);
						if(!$objects){
							$colHtml = '-';
							break;
						}
						foreach(id2pathA($objects, OBJECT_FILES_TABLE) as $id => $path){
							$colHtml .= '<li onclick="openOF(' . $id . ');" title="' . $path . '">' . we_base_util::shortenPath($path, 32) . '</li>';
						}
						$colHtml = '<ul>' . $colHtml . '</ul>';
						break;
					case we_contents_objectBase::TYPE_SELECT:
						$colHtml = $this->getHtmlSelect($col['name'], (string) $this->fields[$col['name']]);
						break;
					case 'input':
						$colHtml = (string) $this->fields[$col['name']];
						break;
					case 'text':
						$colHtml = strip_tags((string) $this->fields[$col['name']]);
						break;
					case 'date':
						$colHtml = $this->fields[$col['name']]->format(g_l('date', '[format][default]'));
						break;
					case 'checkbox':
						$colHtml = g_l('global', ($this->fields[$col['name']] ? '[yes]' : '[no]'));
						break;
					case 'float':
					case 'shopVat':
						$colHtml = (string) round((float) $this->fields[$col['name']], 3);
						break;
				}
			} catch (we_exception_base $e){//in case of an error print error to field
				$colHtml = $e->getMessage();
			}
			$cols[] = new we_html_td($colHtml, ['class' => 'middlefont textCol ' . $col['type']]);
		}

		$this->elementFields = (new we_html_tr($cols))->getHtml();
	}

	protected function processElementGrid(): void{
		// do nothing
	}

	protected function processElementOverlay(): void{
		// do nothing
	}

	protected function getDefaultValues(): array{
		return self::$defaultValues ?: (self::$defaultValues = we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->TableID)));
	}

	protected function getHtmlCheckbox(): string{
		return we_base_permission::hasPerm([we_base_perm::OBJECTFILE_DELETE, we_base_perm::OBJECTFILE_NEW]) &&
			we_base_permission::checkIfRestrictUserIsAllowed($this->ID, OBJECT_FILES_TABLE) ?
			'<input type="checkbox" class="elementSelect" name="publish_docs_' . we_doclist_view::DOCLIST_OBJECTFILES . '" value="' . $this->ID . '_' . OBJECT_FILES_TABLE . '" />' :
			'<i class="fa fa-square-o wecheckIcon disabled"></i>';
	}

	protected function getHtmlIcon(){
		return '<span class="iconListview">' . $this->getHtmlIconThmubnail(64, 140, we_search_view::VIEW_LIST)['imageView'] . '</span>';
	}

	protected function getHtmlShow(): string{
		return $this->Show ? we_html_element::span([
				'class' => 'fa-stack',
				'title' => g_l('modules_objectClassfoldersearch', '[visible_in_ws]')
				], '<i class="fa fa-stack-1x fa-cog"></i>'
			) : we_html_element::span([
				'class' => 'fa-stack',
				'title' => g_l('modules_objectClassfoldersearch', '[not_visible_in_ws]')
				], '<i class="fa fa-stack-2x fa-ban"></i><i class="fa fa-stack-1x fa-cog"></i>'
		);
	}

	protected function getHtmlSearchable(): string{
		return $this->IsSearchable ? we_html_element::span([
				'class' => 'fa-stack',
				'title' => g_l('modules_objectClassfoldersearch', '[issearchable]')
				], '<i class="fa fa-stack-1x fa-search"></i>'
			) : we_html_element::span([
				'class' => 'fa-stack',
				'title' => g_l('modules_objectClassfoldersearch', '[isnotsearchable]')
				], '<i class="fa fa-stack-2x fa-ban"></i><i class="fa fa-stack-1x fa-search"></i>'
		);
	}

	protected function getHtmlFilename(): string{
		$stateclass = !$this->Publish_Date ? 'notpublished' : ((new DateTime($this->Mod_Date)) > (new DateTime($this->Publish_Date)) ? 'changed' : '');
		return we_html_element::a([
				'class' => $stateclass,
				'title' => $this->Path . ' (ID: ' . $this->ID . ')',
				'href' => 'javascript:' . $this->getJsOpenToEdit()
				], we_base_util::shortenPath($this->Text, 32));
	}

	protected function getJsOpenToEdit(): string{
		return "WE().layout.openToEdit('" . $this->docTable . "'," . $this->docID . ",'" . $this->ContentType . "');";
	}

	protected function getHtmlWorkspaces(array $foo): string{
		if(!$foo){
			return '-';
		}
		$paths = id2pathA($foo, FILE_TABLE);
		$out = [];
		foreach($foo as $id){
			if(empty($id)){
				continue;
			}
			$out[] = we_html_element::a([
					'class' => 'middlefont',
					'title' => $paths[$id],
					'href' => "javascript:weSearch.setWs('" . $paths[$id] . "'," . $id . ");",
					], we_base_util::shortenPath($paths[$id], 32));
		}

		return implode('<br/>', $out);
	}

	protected function getHtmlBinary(int $id = 0): string{
		if($id && ($doc = (new DB_WE())->getHash('SELECT Path, ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $id . ' AND IsFolder=0'))){
			return we_html_element::a([
					'class' => 'defaultfont',
					'title' => $doc['Path'],
					'href' => "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE," . $id . ",'" . $doc['ContentType'] . "');",
					], (string) $id);
		}
		return '-';
	}

	protected function getHtmlCollection(int $id = 0): string{
		if($id && ($path = id2path($id, VFILE_TABLE))){
			return we_html_element::a([
					'class' => 'defaultfont',
					'title' => $path,
					'href' => "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.VFILE_TABLE," . $id . ",'" . we_base_ContentTypes::COLLECTION . "');",
					], (string) $id);
		}
		return '-';
	}

	protected function getHtmlLink(string $val): string{
		if(!is_array(($link = we_unserialize($val)))){
			throw new we_exception_base('Field is no link: ' . $val);
		}
		//try{
		if(!($href = we_base_util::shortenPath(we_contents_root::getLinkHref($link, 0, '', false, false, true), 32))){
			return '';
		}
		switch($link['type']){
			case we_base_link::TYPE_INT:
				$id = $link['id'];
				$title = $href . ' (ID: ' . $id . ')';
				$ct = fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $link['id']);
				break;
			case we_base_link::TYPE_OBJ:
				$id = $link['obj_id'];
				$title = $href . ' (Object-ID: ' . $id . ')';
				$ct = we_base_ContentTypes::OBJECT_FILE;
				break;
			default:
				return $href;
		}
		return we_html_element::a([
				'class' => 'defaultfont',
				'title' => $title,
				'href' => "javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.FILE_TABLE," . $id . ",'" . $ct . "');",
				], $href);
	}

	protected function getHtmlSelect(string $name, string $val): string{
		$defaultValues = $this->getDefaultValues();
		if($val !== '' && isset($defaultValues[$name]['meta'][$val])){
			$options = array_combine($defaultValues[$name]['selectKeys'], $defaultValues[$name]['selectValues']);
			$text = $options[$val];
			return (strlen($text) > 20) ? substr($text, 0, 20) . ' &hellip;' : $text;
		}

		return '';
	}

}
