<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_weDocumentCustomerFilter extends we_editor_base{

	private function formWebuser(bool $canChange): string{
		if(!$this->we_doc->WebUserID){
			$this->we_doc->WebUserID = 0;
		}
		$webuser = ''; //g_l('weClass',"[nobody]");

		if($this->we_doc->WebUserID != 0){
			$webuser = id2path($this->we_doc->WebUserID, CUSTOMER_TABLE) ?: '';
//g_l('weClass',"[nobody]");
		}

		if(!$canChange){
			return $webuser;
		}

		$textname = 'wetmp_' . $this->we_doc->Name . '_WebUserID';
		$idname = 'we_' . $this->we_doc->Name . '_WebUserID';

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('Customer')
			->setContentType([])
			->setInput($textname, $webuser, [], false, true)
			->setLabel(g_l('modules_customer', '[connected_with_customer]'))
			->setMaxResults(20)
			->setResult($idname, strval($this->we_doc->WebUserID))
			->setSelector(we_gui_suggest::CustomerSelector)
			->setWidth(434)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_customer_selector',{id:document.we_form.elements." . $idname . ".value,targetID:'" . $idname . "',targetName:'" . $textname . "'});"))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:top.we_cmd('we_customer_edit', document.we_form.elements.yuiAcResultCustomer.value);"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements." . $idname . ".value=0;document.we_form.elements." . $textname . ".value='';_EditorFrame.setEditorIsHot(true);"))
			->setTable(CUSTOMER_TABLE);

		return $weSuggest->getHTML();
	}

	public function show(): string{
		switch($this->we_doc->ContentType){
			case we_base_ContentTypes::IMAGE:
			case we_base_ContentTypes::COLLECTION:
				$html = '';
				break;
			default:
				if(we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER])){
					$filter = $this->we_doc->getDocumentCustomerFilter() ?: new we_customer_documentFilter(FILE_TABLE);
					$view = new we_customer_documentFilterView($filter, 520);
					assert($this->we_doc !== null);
					$view->we_doc = $this->we_doc;
					$html = $view->getFilterHTML();
				} else {
					$html = '';
				}
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('modules_customerFilter', '[customerFilter]'), $html, we_html_multiIconBox::CUSTOMER_FILTER))
				->hide(empty($html))
			)
			->add(new we_html_multiIconBoxContentLine(g_l('modules_customer', '[one_customer]'), $this->formWebuser(we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_CHANGE_DOCS])), we_html_multiIconBox::PROP_CUSTOMER));

		return $this->getPage((!($this->we_doc instanceof we_document_image) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER]) ?
				we_html_element::hidden('we_edit_weDocumentCustomerFilter', 1) : '') .
					(new we_html_multiIconBox($parts, 'weDocProp'))->marginLeft()
					->fold(-1, false, g_l('weClass', '[moreProps]'), g_l('weClass', '[lessProps]'))
					->get()
		);
	}

}
