<?php

/**
 * Simplfied OpenDocument creator class
 *
 */
class we_exim_ExportOpenDocument{
	private const NS = <<<EOT
xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0"
xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
EOT;
	private const MIME = <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<manifest:manifest xmlns:manifest="urn:oasis:names:tc:opendocument:xmlns:manifest:1.0">
<manifest:file-entry manifest:media-type="REPLACE_MIME" manifest:full-path="/"/>
<manifest:file-entry manifest:media-type="text/xml" manifest:full-path="content.xml"/>
<manifest:file-entry manifest:media-type="text/xml" manifest:full-path="meta.xml"/>
<manifest:file-entry manifest:media-type="text/xml" manifest:full-path="styles.xml"/>
</manifest:manifest>
EOT;
	private const META = <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
	<office:document-meta xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0" xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0" office:version="1.0">
	<office:meta>
		<meta:generator>REPLACE_CREATOR</meta:generator>
		<meta:initial-creator>REPLACE_CREATOR</meta:initial-creator>
		<meta:creation-date>REPLACE_TIME</meta:creation-date>
	</office:meta>
</office:document-meta>
EOT;
	private const ODS_HEADER = <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
	<office:document-content REPLACE_NS office:version="1.0">
		<office:automatic-styles>
		 <number:date-style style:name="N84" number:automatic-order="true">
			<number:year number:style="long"/>
			<number:text>-</number:text>
			<number:month number:style="long"/>
			<number:text>-</number:text>
			<number:day number:style="long"/>
		</number:date-style>
		<number:time-style style:name="N43">
			<number:hours number:style="long"/>
			<number:text>:</number:text>
			<number:minutes number:style="long"/>
			<number:text>:</number:text>
			<number:seconds number:style="long"/>
		</number:time-style>
		<number:date-style style:name="N50" number:automatic-order="true" number:format-source="language">
			<number:year number:style="long"/>
			<number:text>-</number:text>
			<number:month number:style="long"/>
			<number:text>-</number:text>
			<number:day number:style="long"/>
			<number:text> </number:text>
			<number:hours number:style="long"/>
			<number:text>:</number:text>
			<number:minutes number:style="long"/>
			<number:text>:</number:text>
			<number:seconds number:style="long"/>
	</number:date-style>
		<style:style style:name="DateCell" style:family="table-cell" style:parent-style-name="Default" style:data-style-name="N84"/>
		<style:style style:name="TimeCell" style:family="table-cell" style:parent-style-name="Default" style:data-style-name="N43"/>
		<style:style style:name="DateTimeCell" style:family="table-cell" style:parent-style-name="Default" style:data-style-name="N50"/>
		<style:style style:name="ce1" style:family="table-cell" style:parent-style-name="Default"><style:text-properties fo:font-weight="bold" style:font-weight-asian="bold" style:font-weight-complex="bold"/></style:style>
	</office:automatic-styles>
<office:body>
	<office:spreadsheet>
EOT;
	private const ODS_FOOTER = <<<EOT
		</office:spreadsheet>
	</office:body>
</office:document-content>
EOT;
	private const ODS_MIME = 'application/vnd.oasis.opendocument.spreadsheet';
	private const HEADER_CELL = '_HEADER_';

	/** @var array<string,string> */
	private $tables = [];

	/** @var string */
	private $openTable = '';
	private $db;
	private $empty = true;

	public function __construct(){
		$this->db = new DB_WE();
	}

	/**
	 * Minimalistic creator of OASIS OpenDocument
	 *
	 * @return string  OASIS OpenDocument data
	 *
	 * @access  public
	 */
	private function create(string $file): void{
		$zip = new ZipArchive();
		$zip->open($file, ZIPARCHIVE::OVERWRITE);
		$zip->addFromString('mimetype', self::ODS_MIME);
		$zip->addFromString('content.xml',
											strtr(self::ODS_HEADER, ['REPLACE_NS' => self::NS]) .
			$this->getContent() .
			strtr(self::ODS_FOOTER, ['REPLACE_NS' => self::NS]));
		$zip->addFromString('meta.xml', strtr(self::META, ['REPLACE_CREATOR' => 'webEdition ' . WE_VERSION, 'REPLACE_TIME' => strftime('%Y-%m-%dT%H:%M:%S')])); //@phan-suppress-current-line PhanParamSuspiciousOrder
		//$zip->addFromString('styles.xml', strtr(self::STYLE, ['REPLACE_NS' => self::NS]));
		$zip->addFromString('META-INF/manifest.xml', strtr(self::MIME, ['REPLACE_MIME' => self::ODS_MIME]));
		$zip->close();
	}

	private function createTemp(string $outPutFilename, bool $persistent): bool{
		if($persistent){
			if(we_base_file::save($outPutFilename, ' ')){
				self::create($outPutFilename);
				return true;
			}
			return false;
		}
		$tmpFile = we_base_file::saveTemp(' ');

		self::create($tmpFile);
		header('Content-Type: ' . self::ODS_MIME, true);
		header('Pragma: public');
		header('Expires: 0');
		header('Cache-control: private, max-age=0, must-revalidate');
		header('Content-Disposition: attachment; filename="' . trim(htmlentities(basename($outPutFilename))) . '"');
		header('Content-Description: ' . trim(htmlentities(basename($outPutFilename))));
		header('Content-Length: ' . filesize($tmpFile));
		readfile($tmpFile);
		we_base_file::delete($tmpFile);
		return true;
	}

	private function getContent(): string{
		$this->closeTable();
		return implode('', $this->tables);
	}

	private function closeTable(): void{
		if($this->openTable){
			$this->tables[$this->openTable] .= '</table:table>';
			$this->openTable = '';
		}
	}

	private function addTable(string $name): void{
		$this->closeTable();
		$this->openTable = $name;
		$this->tables[$this->openTable] = '<table:table table:name="' . htmlspecialchars($name) . '">';
	}

	private function addRow(string $row): void{
		$this->tables[$this->openTable] .= '<table:table-row>' . $row . '</table:table-row>';
	}

	private static function addCell(string $type, $val): string{
		$val = is_string($val) ? mb_convert_encoding($val, 'UTF-8', 'UTF-8,ISO-8859-15') : $val;
		switch($type){
			case we_database_base::TYPE_NULL:
				$val = '';
			case self::HEADER_CELL:
				return '<table:table-cell table:style-name="ce1" office:value-type="string" calcext:value-type="string"><text:p>' . htmlspecialchars(stripslashes((string) $val)) . '</text:p></table:table-cell>';
			default:
			case we_database_base::TYPE_SHORT_STRING:
				return '<table:table-cell office:value-type="string" calcext:value-type="string"><text:p>' . htmlspecialchars(stripslashes((string) $val)) . '</text:p></table:table-cell>';
			case we_database_base::TYPE_SHORT_NUMBER:
				return '<table:table-cell office:value-type="float" office:value="' . $val . '"><text:p>' . strval($val) . '</text:p></table:table-cell>';
			case we_database_base::TYPE_SHORT_TIME:
				return '<table:table-cell office:value-type="time" calcext:value-type="time" office:date-value="' . date('\P\TH\Hi\Ms\S', (int) (strtotime((string) $val) ?? 0)) . '" table:style-name="TimeCell"><text:p>' . ($val) . '</text:p></table:table-cell>';
			case we_database_base::TYPE_SHORT_DATE:
				return '<table:table-cell office:value-type="date" calcext:value-type="date" office:date-value="' . date('Y-m-d', (int) (strtotime((string) $val) ?? 0)) . '" table:style-name="DateCell"><text:p>' . ($val) . '</text:p></table:table-cell>';
			case we_database_base::TYPE_SHORT_DATETIME:
				return '<table:table-cell office:value-type="date" calcext:value-type="date" office:date-value="' . date('Y-m-d\TH:i:s', (int) (strtotime((string) $val) ?? 0)) . '" table:style-name="DateTimeCell"><text:p>' . ($val) . '</text:p></table:table-cell>';
		}
	}

	/**
	 * @param string $tableName
	 * @param string $query
	 * @param string[] $skipFields
	 * @param bool $union
	 * @return bool
	 */
	public function addTabQuery(string $tableName, string $query, array $skipFields = [], bool $union = false): bool{
		$this->db->query($query, $union, true);

		$header = $this->db->fieldNames();
		'@phan-var array<int,string> $header';
		if(empty($header)){
			return false;
		}
		$this->addTable(mb_convert_encoding($tableName, 'UTF-8', 'ISO-8859-15,UTF-8') ?: '');
		$this->empty = false;
		$row = '';
		$fieldTypes = [];
		foreach($header as $pos => $name){
			if(in_array($name, $skipFields, true)){
				continue;
			}
			$row .= self::addCell(self::HEADER_CELL, $name);
			$fieldTypes[$name] = $this->db->field_type_short($pos);
		}
		$this->addRow($row);
		while($this->db->next_record()){
			$row = '';
			foreach($this->db->getRecord() as $name => $val){
				if(in_array($name, $skipFields, true)){
					continue;
				}
				$row .= self::addCell($fieldTypes[$name], $val) . "\n";
			}
			$this->addRow($row);
		}
		return true;
	}

	/* public function addTabEAVQuery(string $tableName, string $query, array $keyFields, array $repeatFields, array $repeatHeaders, array $skipFields = [], bool $union = false): bool{
	  $this->db->query($query, $union, true);
	  $header = $this->db->fieldNames();
	  '@phan-var array<int,string> $header';
	  if(empty($header)){
	  return false;
	  }
	  $this->addTable(mb_convert_encoding($tableName, 'UTF-8', 'ISO-8859-15,UTF-8') ?: '');
	  $this->empty = false;
	  $row = '';
	  $fieldTypes = [];
	  foreach($header as $pos => $name){
	  if(in_array($name, $skipFields, true)){
	  continue;
	  }
	  if(!in_array($name, $repeatFields, true)){
	  $row .= self::addCell(self::HEADER_CELL, $name);
	  }
	  $fieldTypes[$name] = $this->db->field_type_short($pos);
	  }
	  foreach($repeatHeaders as $name){
	  $row .= self::addCell(self::HEADER_CELL, $name);
	  }
	  $this->addRow($row);
	  $oldKeys= array_flip($keyFields);
	  while($this->db->next_record()){
	  $row = '';
	  }
	  } */

	/**
	 * @return void
	 * @throws Exception
	 */
	public function getFile(string $filename, bool $persistent = false): void{
		if($this->empty){
			throw new Exception('no content');
		}
		if(!$this->createTemp($filename, $persistent)){
			throw new Exception('error creating ods');
		}
	}

}
