<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aacab26fe68a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 10:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_rule extends we_base_model implements we_exim_xmlIF{
	public const BACKUP_XML_TAG = 'we:navigationrule';

	/** @var string */
	var $Table = NAVIGATION_RULE_TABLE;

	/** @var string */
	public $ContentType = we_base_ContentTypes::NAVIGATIONRULE; //for backup

	/** @var int */
	var $ID;

	/** @var string */
	var $NavigationName;

	/** @var int */
	var $NavigationID;

	/** @var string */
	var $SelectionType;

	/** @var int */
	var $FolderID;

	/** @var int */
	var $DoctypeID;

	/** @var int */
	var $ClassID;

	/** @var int[] */
	var $Categories = [];

	/** @var int */
	var $WorkspaceID;

	/** @var string */
	var $Href;

	/** @var bool */
	var $SelfCurrent;

	/** @var array<string,string> */
	var $persistent_slots = [
		'ID' => we_base_request::INT,
		'NavigationName' => we_base_request::STRING,
		'NavigationID' => we_base_request::INT,
		'SelectionType' => we_base_request::STRING,
		'FolderID' => we_base_request::INT,
		'DoctypeID' => we_base_request::INT,
		'ClassID' => we_base_request::INT,
		'Categories' => we_base_request::INTLISTA,
		'WorkspaceID' => we_base_request::INT
	];

	public function __construct(int $id = 0){
		parent::__construct(NAVIGATION_RULE_TABLE);
		if(!$id){
			return;
		}
		$this->initByID($id);
	}

	function initByID(int $ruleId): bool{
		return parent::load($ruleId);
	}

	/**
	 *
	 * @param string $navigationName
	 * @param int $navigationId
	 * @param string $selectionType
	 * @param int $folderId
	 * @param int $doctype
	 * @param int $classId
	 * @param int[] $categories
	 * @param int $workspaceId
	 * @param string $href
	 * @param bool $selfCurrent
	 * @return we_navigation_rule
	 */
	static function getWeNavigationRule(string $navigationName, int $navigationId, string $selectionType, int $folderId, int $doctype, int $classId, array $categories, int $workspaceId, string $href = '', bool $selfCurrent = true): self{
		$rule = new self();
		$rule->NavigationName = $navigationName;
		$rule->NavigationID = $navigationId;
		$rule->SelectionType = $selectionType;
		$rule->FolderID = $folderId;
		$rule->DoctypeID = $doctype;
		$rule->ClassID = $classId;
		$rule->Categories = $categories;
		$rule->WorkspaceID = $workspaceId;

		$rule->Href = $href;
		$rule->SelfCurrent = $selfCurrent;

		return $rule;
	}

	function we_load(int $id): bool{
		return $this->load($id);
	}

	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		$ret = parent::save($this->ID ? false : true);
		if($ret){
			we_navigation_cache::saveRules(we_navigation_ruleControl::getAllNavigationRules());
			we_navigation_cache::delNavigationTree($this->NavigationID);
		}
		return $ret;
	}

	function delete(): bool{
		$ret = parent::delete();
		if($ret){
			we_navigation_cache::saveRules(we_navigation_ruleControl::getAllNavigationRules());
			we_navigation_cache::delNavigationTree($this->NavigationID);
		}
		return $ret;
	}

	function processVariables(): void{
		if(($name = we_base_request::string('CategoriesControl'))){
			$categories = [];
			$i = 0;
			while(($cat = we_base_request::string($name . '_variant0_' . $name . '_item' . $i))){
				$categories[] = $cat;
				$i++;
			}

			$this->Categories = path_to_idA($categories, CATEGORY_TABLE);
		}

		foreach($this->persistent_slots as $key => $type){
			if(($tmp = we_base_request::_($type, $key, false)) !== false){
				$this->{$key} = $tmp;
			}
		}

		$this->isnew = ($this->ID == 0);
	}

	public function toXML(array $attribs = [], array $extraSlots = [], string $content = ''): string{
		return parent::toXML($attribs, ['ContentType' => we_base_request::STRING]);
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
