<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e5bf7f91c9c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 13:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_mediaDocsDelete extends we_rpc_cmd{

	function execute(): we_rpc_response{
		if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE])){
			//return 'no perms';
		}

		$selectedItems = [];
		$csv = we_base_request::string('we_cmd', '', 0);
		if($csv){
			$allDocs = explode(',', $csv);
			foreach($allDocs as $v){
				$teile = explode('_', $v, 2);
				$selectedItems[$teile[1]][] = intval($teile[0]);
			}
		}

		$this->resp->setData('response', $this->delete($selectedItems[FILE_TABLE]));

		return $this->resp;
	}

	function delete(array $selectedItems = []): array{
		if(!$selectedItems){
			return ['message' => g_l('alert', '[nothing_to_delete]'), 'rewriteMain' => false];
		}

		$db = DB_WE::$inst;

		// check if docs are media documents and not protected
		$docsToDelete = array_filter($db->getAllFirstq('SELECT ID,ContentType,IsProtected,ParentID,Path FROM ' . FILE_TABLE . ' WHERE ID IN (' . (implode(',', $selectedItems)) . ')', true), function($var){
			return !$var[1] && in_array($var[0], [
					we_base_ContentTypes::IMAGE,
					we_base_ContentTypes::AUDIO,
					we_base_ContentTypes::VIDEO,
					we_base_ContentTypes::FLASH,
					we_base_ContentTypes::APPLICATION,
					we_base_ContentTypes::PDF], true
			);
		});

		// check if docs are used
		if($docsToDelete){
			foreach($db->getAllq('SELECT remObj FROM ' . FILELINK_TABLE . ' WHERE type="media" AND remTable="' . stripTblPrefix(FILE_TABLE) . '" AND remObj IN (' . implode(',', array_keys($docsToDelete)) . ') AND position=0', true) as $v){
				unset($docsToDelete[$v]);
			}
		}

		// check owner restrictions and try to delete
		foreach(array_keys($docsToDelete) as $k){
			if(we_base_permission::checkIfRestrictUserIsAllowed((int) $k, FILE_TABLE)){
				we_base_delete::deleteEntry((int) $k, FILE_TABLE);
			}
		}

		if(we_base_delete::$deletedItems){
			if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
				we_customer_documentFilter::deleteModel(we_base_delete::$deletedItems, FILE_TABLE);
			}
			we_base_history::deleteFromHistory(we_base_delete::$deletedItems, FILE_TABLE);

			return ['message' => g_l('alert', '[delete][ok]'), 'rewriteMain' => true, 'deletedItems' => we_base_delete::$deletedItems];
		}
		return ['message' => g_l('alert', '[nothing_to_delete]'), 'rewriteMain' => false];
	}

}
