<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: def1d23a4935 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-15 13:13 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_delete extends we_selector_file{

	function __construct(int $id, string $table = FILE_TABLE){
		parent::__construct($id, $table);
		$this->title = g_l('fileselector', '[delSelector][title]');
	}

	function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		switch($what){
			case self::DEL:
				$this->printDoDelEntryHTML();
				break;
			default:
				parent::printHTML($what);
		}
	}

	protected function setFramesetJavaScriptOptions(): void{
		parent::setFramesetJavaScriptOptions();
		$this->jsoptions['options']['seemForOpenDelSelector'] = intval($_SESSION['weS']['seemForOpenDelSelector']['ID'] ?? 0);
		unset($_SESSION['weS']['seemForOpenDelSelector']['ID']);
	}

	protected function printCmdHTML(): void{
		$this->jsCmd->addCmd('setButtons', [['DelBut', $this->dir ? true : false]]);
		parent::printCmdHTML();
	}

	/* function renameChildrenPath(int $id): void{
	  $db = DB_WE::$inst;
	  $parentPath = fS('SELECT Path FROM ' . $db->escape($this->table) . ' WHERE ID=' . intval($id));
	  $db->query('UPDATE ' . $db->escape($this->table) . ' SET Path=CONCAT("' . $parentPath . '/",Text) WHERE ParentID=' . intval($id));
	  $all = $db->getAllq('SELECT ID FROM ' . $db->escape($this->table) . ' WHERE IsFolder=1 AND ParentID=' . intval($id), true);
	  foreach($all as $id){
	  $this->renameChildrenPath((int) $id);
	  }
	  } */

	function printDoDelEntryHTML(): void{
		if(($del = we_base_request::int("todel"))){
			$_SESSION['weS']['fragDel'] = [
				'todel' => $del,
				'we_not_deleted_entries' => [],
				'we_go_seem_start' => false,
			];
			$this->jsCmd->addCmd('we_cmd', ['del_frag', $del]);
			$this->jsCmd->addCmd('close');
		}
		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function printFooterTable(): string{
		if($this->values['Text'] === '/'){
			$this->values['Text'] = '';
		}

		return '
<table id="footer" class="defaultfont">
	<tr>
		<td class="description">' . g_l('fileselector', '[filename]') . '</td>
		<td style="text-align:left">' . we_html_tools::textInput('fname', (string) $this->values['Text'], 0, ['style' => "width:100%", 'readonly' => "readonly"]) . '</td>
	</tr>
</table><div id="footerButtons">' . we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::DELETE, "javascript:if(document.we_form.fname.value==''){WE().util.dialogTop(window).exit_close();}else{WE().util.dialogTop(window).deleteEntry();}", '', "", true, false), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).exit_close();")) . '</div>';
	}

	protected function query(): array{
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . intval($this->dir) . ' AND((1' . we_users_util::makeOwnersSql() . ')' .
						self::getWsQuery($this->table, false) . ')' . ($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : '')
		);
	}

}
