<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0592961c439f $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-27 14:59 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_statusMails{

	/** @var array<string,bool> */
	var $FieldsHidden; //an array of statusfield names not to be shown in order

	/** @var array<string,bool> */
	var $FieldsHiddenCOV; //an array of statusfield names not to be shown in order

	/** @var array<string,string> */
	var $FieldsText; //an array with keys equal to name of statusfield, and value = text to be shown

	/** @var array<string,bool> */
	var $FieldsMails; //an array with keys equal to name of statusfield, and value = 0 for no Mail, 1 for Mail by Hand, 2 for automatic mails

	/** @var array<string,string> */
	var $EMailData; // an array with the E-Mail data, see getShopStatusMails

	/** @var array<string,mixed> */
	var $LanguageData; // an array with the Language data, see getShopStatusMails

	/** @var array<string,array<string,mixed>> */
	var $FieldsDocuments; // an array with dfault values and separate Arrays for each Langauge, see getShopStatusMails

	/** @var string[] */
	public const StatusFields = ['DateOrder', 'DateConfirmation', 'DateCustomA', 'DateCustomB', 'DateCustomC', 'DateShipping', 'DateCustomD', 'DateCustomE', 'DatePayment',
		'DateCustomF', 'DateCustomG', 'DateCancellation', 'DateCustomH', 'DateCustomI', 'DateCustomJ', 'DateFinished'];

	/** @var string[] */
	public const BaseDateFields = ['DateOrder', 'DateConfirmation', 'DateShipping', 'DatePayment', 'DateCancellation', 'DateFinished'];

	/** @var string[] */
	public const MailFields = ['MailShipping', 'MailPayment', 'MailOrder', 'MailConfirmation', 'MailCustomA', 'MailCustomB', 'MailCustomC', 'MailCustomD', 'MailCustomE',
		'MailCustomF', 'MailCustomG', 'MailCustomH', 'MailCustomI', 'MailCustomJ', 'MailCancellation', 'MailFinished'];

	function __construct(array $FieldsHidden, array $FieldsHiddenCOV, array $FieldsText, array $FieldsMails, array $EMailData, array $LanguageData, array $FieldsDocuments){
		$this->FieldsHidden = $FieldsHidden;
		$this->FieldsHiddenCOV = $FieldsHiddenCOV;
		$this->FieldsText = $FieldsText;
		$this->FieldsMails = $FieldsMails;
		$this->EMailData = $EMailData;
		$this->LanguageData = $LanguageData;
		$this->FieldsDocuments = $FieldsDocuments;
	}

	public static function initByRequest(): self{//FIXME: this is unchecked!!
		return new self(
				we_base_request::boolList('FieldsHidden'),
				we_base_request::boolList('FieldsHiddenCOV'),
				we_base_request::stringList('FieldsText'),
				we_base_request::boolList('FieldsMails'),
				we_base_request::stringList('EMailData'),
				we_base_request::stringList('LanguageData'),
				we_base_request::intList('FieldsDocuments')
		);
	}

	static function getShopStatusMails(): self{
		$documentsarray = [
			'default' => [
				'DateOrder' => 0,
				'DateConfirmation' => 0,
				'DateCustomA' => 0,
				'DateCustomB' => 0,
				'DateCustomC' => 0,
				'DateShipping' => 0,
				'DateCustomD' => 0,
				'DateCustomE' => 0,
				'DateCancellation' => 0,
				'DateCustomF' => 0,
				'DateCustomG' => 0,
				'DatePayment' => 0,
				'DateCustomH' => 0,
				'DateCustomI' => 0,
				'DateCustomJ' => 0,
				'DateFinished' => 0,
			]
		];
		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		unset($lcvalue);
		foreach($frontendL as $langkey){
			$documentsarray[$langkey] = $documentsarray['default'];
		}
		if(($zw2 = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="weShopStatusMails"'))){
			$zw2 = we_unserialize($zw2);
			$zw2['FieldsHidden'] = we_base_request::filterBoolList($zw2['FieldsHidden']);
			$zw2['FieldsHiddenCOV'] = we_base_request::filterBoolList($zw2['FieldsHiddenCOV']);
			$zw2['FieldsMails'] = we_base_request::filterBoolList($zw2['FieldsMails']);
		} else {
			$zw2 = [
				'FieldsHidden' => [],
				'FieldsHiddenCOV' => [],
				'FieldsText' => [],
				'FieldsMails' => [],
				'EMailData' => [],
				'LanguageData' => [],
				'FieldsDocuments' => []
			];
		}
		return new self(
				array_merge([//Fieldshidden
					'DateOrder' => false,
					'DateConfirmation' => true,
					'DateCustomA' => true,
					'DateCustomB' => true,
					'DateCustomC' => true,
					'DateShipping' => false,
					'DateCustomD' => true,
					'DateCustomE' => true,
					'DateCancellation' => true,
					'DateCustomF' => true,
					'DateCustomG' => true,
					'DatePayment' => false,
					'DateCustomH' => true,
					'DateCustomI' => true,
					'DateCustomJ' => true,
					'DateFinished' => true
						], $zw2['FieldsHidden']), array_merge([//FieldshiddenCOV
					'DateOrder' => false,
					'DateConfirmation' => true,
					'DateCustomA' => true,
					'DateCustomB' => true,
					'DateCustomC' => true,
					'DateShipping' => false,
					'DateCustomD' => true,
					'DateCustomE' => true,
					'DateCancellation' => true,
					'DateCustomF' => true,
					'DateCustomG' => true,
					'DatePayment' => false,
					'DateCustomH' => true,
					'DateCustomI' => true,
					'DateCustomJ' => true,
					'DateFinished' => true
						], $zw2['FieldsHiddenCOV']), array_merge([//FieldsTexts
					'DateOrder' => g_l('modules_shop', '[bestelldatum]'),
					'DateConfirmation' => g_l('modules_shop', '[bestaetigt]'),
					'DateCustomA' => g_l('modules_shop', '[customA]'),
					'DateCustomB' => g_l('modules_shop', '[customB]'),
					'DateCustomC' => g_l('modules_shop', '[customC]'),
					'DateShipping' => g_l('modules_shop', '[bearbeitet]'),
					'DateCustomD' => g_l('modules_shop', '[customD]'),
					'DateCustomE' => g_l('modules_shop', '[customE]'),
					'DatePayment' => g_l('modules_shop', '[bezahlt]'),
					'DateCustomF' => g_l('modules_shop', '[customF]'),
					'DateCustomG' => g_l('modules_shop', '[customG]'),
					'DateCancellation' => g_l('modules_shop', '[storniert]'),
					'DateCustomH' => g_l('modules_shop', '[customH]'),
					'DateCustomI' => g_l('modules_shop', '[customI]'),
					'DateCustomJ' => g_l('modules_shop', '[customJ]'),
					'DateFinished' => g_l('modules_shop', '[beendet]')
						], $zw2['FieldsText']), array_merge([//FieldsMails
					'DateOrder' => true,
					'DateConfirmation' => true,
					'DateCustomA' => true,
					'DateCustomB' => true,
					'DateCustomC' => true,
					'DateShipping' => true,
					'DateCustomD' => true,
					'DateCustomE' => true,
					'DateCancellation' => true,
					'DateCustomF' => true,
					'DateCustomG' => true,
					'DatePayment' => true,
					'DateCustomH' => true,
					'DateCustomI' => true,
					'DateCustomJ' => true,
					'DateFinished' => false
						], $zw2['FieldsMails']), array_merge([//EMailData
					'address' => '',
					'name' => '',
					'bcc' => '',
					'DocumentSubjectField' => 'Title',
					'DocumentAttachmentFieldA' => '',
					'DocumentAttachmentFieldB' => '',
					'emailField' => '',
					'titleField' => ''
						], $zw2['EMailData']), array_merge([//LanguageData
					'useLanguages' => true,
					'languageField' => '',
					'languageFieldIsISO' => false
						], $zw2['LanguageData']), array_merge($documentsarray, $zw2['FieldsDocuments'])
		);
	}

	function sendEMail(string $was, int $orderid, array $cdata, string $pagelang = ''): bool{
		$DB_WE = we_database_base::$inst;
		$recipientOK = (!empty($this->EMailData['emailField']) && !empty($cdata[$this->EMailData['emailField']]) && we_check_email($cdata[$this->EMailData['emailField']]));
		$UserLang = '';
		if(!empty($this->LanguageData['useLanguages']) && !empty($this->LanguageData['languageField']) && !empty($cdata[$this->LanguageData['languageField']])){
			if($pagelang && !empty($this->FieldsDocuments[$pagelang]['Date' . $was])){
				$docID = intval($this->FieldsDocuments[$pagelang]['Date' . $was]);
				$field = 1;
			} else {
				$docID = intval(isset($this->FieldsDocuments[$cdata[$this->LanguageData['languageField']]]) && isset($this->FieldsDocuments[$cdata[$this->LanguageData['languageField']]]['Date' . $was]) ?
						$this->FieldsDocuments[$cdata[$this->LanguageData['languageField']]]['Date' . $was] :
						$this->FieldsDocuments['default']['Date' . $was]);
				$field = 2;
			}
			if(!empty($this->LanguageData['languageField']) && !empty($cdata[$this->LanguageData['languageField']])){
				$UserLang = $cdata[$this->LanguageData['languageField']];
			}
		} else {
			$docID = intval($this->FieldsDocuments['default']['Date' . $was]);
			$field = 3;
			if(!empty($this->LanguageData['languageField']) && !empty($cdata[$this->LanguageData['languageField']])){
				$UserLang = $cdata[$this->LanguageData['languageField']];
			}
		}

		if(!$docID || !we_base_file::isWeFile($docID, FILE_TABLE)){
			t_e('Document to send as status mail is empty! ID: ' . $docID, $field);
			return false;
		}

		$_SESSION['WE_SendMail'] = true;
		$_REQUEST['we_orderid'] = $orderid;
		$_REQUEST['we_userlanguage'] = $UserLang;
		$_REQUEST['we_shopstatus'] = $was;
		$codes = we_getDocumentByID($docID, '');
		$maildoc = new we_document_webEdition($docID);

		if(!empty($this->EMailData['DocumentAttachmentFieldA'])){
			$attachmentA = $maildoc->getElementS($this->EMailData['DocumentAttachmentFieldA']);
			$codes = $codes . $attachmentA;
		}
		unset($_REQUEST['we_orderid']);
		we_tag_sendMail::$active = false;

		$subject = $maildoc->getElementS($this->EMailData['DocumentSubjectField']) ?: 'no subject given';
		if($recipientOK && $this->EMailData['address'] != '' && we_check_email($this->EMailData['address'])){
			$from = new we_mail_address($this->EMailData['address'], $this->EMailData['name']);

			$phpmail = new we_mail_mail([new we_mail_address((string) $cdata[$this->EMailData['emailField']], ( (!empty($this->EMailData['titleField']) && !empty($cdata[$this->EMailData['titleField']]) ) ? $cdata[$this->EMailData['titleField']] . ' ' : '') . $cdata['Forename'] . ' ' . $cdata['Surname'])], $subject, $from);
			$phpmail->setIsEmbedImages(true);
			$phpmail->addHTMLPart($codes);
			$phpmail->setTextPartOutOfHTML($codes);
			if(isset($this->EMailData['bcc']) && $this->EMailData['bcc'] != ''){
				$bccArray = explode(',', $this->EMailData['bcc']);
				$phpmail->setBCC($bccArray);
			}
			if(!empty($this->EMailData['DocumentAttachmentFieldA'])){
				$attachmentinternal = $maildoc->getElementI($this->EMailData['DocumentAttachmentFieldA'] . we_base_link::MAGIC_INT_LINK);
				$attachment = $maildoc->getElementI($this->EMailData['DocumentAttachmentFieldA'] . ($attachmentinternal ? we_base_link::MAGIC_INT_LINK_ID : ''));
				if($attachment){
					$phpmail->doaddAttachment(WE_DOCUMENT_ROOT . ($attachmentinternal ? id2path($attachment) : $attachment));
				}
			}
			if(!empty($this->EMailData['DocumentAttachmentFieldB'])){
				$attachmentBinternal = $maildoc->getElementS($this->EMailData['DocumentAttachmentFieldB'] . we_base_link::MAGIC_INT_LINK);
				$attachmentB = $maildoc->getElementI($this->EMailData['DocumentAttachmentFieldB'] . ($attachmentBinternal ? we_base_link::MAGIC_INT_LINK_ID : ''));
				if($attachmentB){
					$phpmail->doaddAttachment(WE_DOCUMENT_ROOT . ($attachmentBinternal ? id2path($attachmentB) : $attachmentB));
				}
			}
			$phpmail->buildMessage();
			if($phpmail->send()){
				$DB_WE->query('REPLACE INTO ' . SHOP_ORDER_DATES_TABLE . ' SET type="Mail' . $DB_WE->escape($was) . '", ID=' . $orderid);

				return true;
			}
		}
		return false;
	}

	function checkAutoMailAndSend(string $was, int $orderid, array $cdata, string $lang): void{
		if($this->FieldsMails['Date' . $was] == 2){
			$this->sendEMail($was, $orderid, $cdata, $lang);
		}
	}

	function getEMailHandlerCode(string $was, string $dateSet): string{
		if(!$this->FieldsMails['Date' . $was]){
			return '';
		}
		$datetimeform = "00.00.0000 00:00";
		$dateform = "00.00.0000";
		$EMailhandler = '<td>' . g_l('modules_shop', '[statusmails][EMail]') . ': </td>';

		if(($m = we_base_request::string("Mail" . $was)) && $m != $datetimeform){
			$EMailhandler .= '<td style="width:150px;">' . $m . '</td>';
			$but = we_html_button::create_button('fa:mail_resend,fa-lg fa-envenlope,fa-lg fa-rotate-right', "javascript:check=confirm('" . g_l('modules_shop', '[statusmails][resent]') . "'); if (check){SendMail('" . $was . "');}");
		} else {
			$EMailhandler .= '<td style="width:150px">&nbsp;</td>';
			$but = we_html_button::create_button(we_html_button::SEND, "javascript:SendMail('" . $was . "')");
		}
		return $EMailhandler . '<td>' . ($dateSet != $dateform ? $but : '') . '</td>';
	}

	function save(): bool{
		$DB_WE = DB_WE::$inst;

		if($DB_WE->query('REPLACE ' . SETTINGS_TABLE . ' SET pref_value="' . $DB_WE->escape(we_serialize((array) $this, SERIALIZE_JSON)) . '",tool="shop",pref_name="weShopStatusMails"')){
			$CLFields = we_unserialize(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_CountryLanguage"'));
			if(!$CLFields){
				$CLFields = [
					'languageField' => $this->LanguageData['languageField'],
					'languageFieldIsISO' => $this->LanguageData['languageFieldIsISO']];
				$DB_WE->query('REPLACE ' . SETTINGS_TABLE . ' SET pref_value="' . $DB_WE->escape(we_serialize($CLFields, SERIALIZE_JSON)) . '",tool="shop",pref_name="shop_CountryLanguage"');
			}

			return true;
		}
		return false;
	}

}
