<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e231b5b90bce $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-09 11:00 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_view extends we_modules_view{
	public const CLFieldsDefault = [
		'stateField' => '-',
		'stateFieldIsISO' => 0,
		'languageField' => '-',
		'languageFieldIsISO' => 0
	];

	/** @var array{stateField:string,stateFieldIsISO:int,languageField:string,languageFieldIsISO:int} */
	private $CLFields = self::CLFieldsDefault;

	function getJSTop(): string{
		$modData = we_modules_info::getModuleField(we_modules_info::SHOP, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		/// config
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = $feldnamen[$i] ?? '';
		}

		$fe = explode(',', $feldnamen[3]);
		$classid = $fe[0];
		//$resultO = count ($fe);
		$resultO = array_shift($fe);

		// whether the resultset is empty?
		$resultD = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view.js', '', ['id' => 'loadVarShop_view', 'data-viewData' => setDynamicVar([
					'isDocument' => intval($resultD),
					'isObject' => ((!empty($resultO))),
					'classID' => intval($classid),
					'title' => $title,
		])]);
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_property.js');
	}

	function getProperties(we_base_jsCmd $jscmd): string{
		$weShopStatusMails = we_shop_statusMails::getShopStatusMails();
		$hiddenStatusFields = array_keys(array_filter($weShopStatusMails->FieldsHidden, function ($v): bool{
				return $v == 1;
			}));

		// Get Country and Langfield Data
		$this->CLFields = we_unserialize(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_CountryLanguage"'), self::CLFieldsDefault);

		// config
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		$waehr = '&nbsp;' . htmlspecialchars($feldnamen[0]);
		//$numberformat = $feldnamen[2];
		$mwst = floatval($feldnamen[1] ?: 0);

		if(!in_array(we_base_request::string('cmd'), ['save_order_customer',
				'save_shipping_cost'])){
			$this->processCommands($jscmd); // FIXME: this call should be obsolete here! Check for other cmd's
		}

		$bid = we_base_request::int('bid', 0);
		if(we_base_request::bool('deletethisorder')){
			$this->db->query('DELETE FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
			$this->db->query('DELETE FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid);
			$this->db->query('DELETE FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid);
			$jscmd->addCmd('deleteTreeEntry', $bid);
			return we_html_tools::getHtmlTop('', $jscmd->getCmds(), we_html_element::body(['class' => "weEditorBody",
						'onunload' => "doUnload()"], '<table style="width:300px">
			  <tr>
				<td colspan="2" class="defaultfont">' . we_html_tools::dialogLayout('<span class="defaultfont">' . g_l('modules_shop', '[geloscht]') . '</span>', g_l('modules_shop', '[loscht]')) . '</td>
			  </tr>
			  </table>'));
		}

		if(($id = we_base_request::int('deleteaarticle'))){
			$this->db->query('DELETE FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $id);
			if(fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid) < 1){
				//last item deleted, delete the order itself
				$this->db->query('DELETE FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
				$this->db->query('DELETE FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid);
				$jscmd->addCmd('deleteTreeEntry', $bid);
				$jscmd->addCmd('showDeleteOrder', $bid);

				return we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
								'bid' => $bid,
						])]) .
						$jscmd->getCmds(), we_html_element::body());
			}
		}

		// Get Customer data
		list('customerID' => $_REQUEST['cid'], 'customerLang' => $lang) = getHash('SELECT customerID,customerLang FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);

		if(!($fields = we_unserialize(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="edit_shop_properties"')))){
			// we have an array with following syntax:
			// array ( 'customerFields' => array('fieldname ...',...)
			//         'orderCustomerFields' => array('fieldname', ...) )
			//unsupported
			t_e('unsupported Shop-Settings found. Please open settings and readjust the settings of the shop module.');
			$fields = [];
		}

		$customer = $this->getOrderCustomerData($bid, $fields ? $fields['orderCustomerFields'] : []);

		if(($sendMail = we_base_request::string('SendMail'))){
			$weShopStatusMails->sendEMail($sendMail, $bid, $customer);
		}
		foreach(we_shop_statusMails::StatusFields as $field){
			if(($val = we_base_request::string($field))){
				$DateOrder = new DateTime($val);
				if(in_array($field, we_shop_statusMails::BaseDateFields, true)){
					$this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . $field . '="' . $this->db->escape($DateOrder->format(we_database_base::DATE_TIME_FORMAT)) . '" WHERE ID=' . $bid);
				} else {
					$this->db->query('REPACE INTO ' . SHOP_ORDER_DATES_TABLE . ' SET ID=' . $bid . ',type="' . $field . '",date="' . $this->db->escape($DateOrder->format(we_database_base::DATE_TIME_FORMAT)));
				}
				$weShopStatusMails->checkAutoMailAndSend(substr($field, 4) ?: '', $bid, $customer, $lang);
			}
		}

		if(($article = we_base_request::int('article'))){
			if(($preis = we_base_request::float('preis', -1)) >= 0){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET Price=' . abs($preis) . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			} else if(($anz = we_base_request::float('anzahl', -1)) >= 0){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET quantity=' . abs($anz) . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			} else if(($vat = we_base_request::float('vat', -1)) >= 0){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET Vat="' . abs($vat) . '" WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			}
		}

		// order has still articles - get them all
		// ********************************************************************************
		// first get all information about orders, we need this for the rest of the page
		//
		$showBaseFields = array_diff(we_shop_statusMails::BaseDateFields, $hiddenStatusFields);
		$showDateFields = array_diff(we_shop_statusMails::StatusFields, $hiddenStatusFields);
		$showMailFields = array_diff(we_shop_statusMails::MailFields, $hiddenStatusFields);
		$showAdvanced = array_merge(array_diff($showDateFields, $showBaseFields), $showMailFields);

		$allBaseDates = [];
		foreach($showBaseFields as $cur){
			$allBaseDates[] = 'DATE_FORMAT(' . $cur . ',"' . g_l('date', '[format][mysql]') . '") AS ' . $cur;
		}

		$orderData = getHash('SELECT customerID,customOrderNo,pricesNet,calcVat,shippingCost,shippingNet,shippingVat,customFields,' . implode(',', $allBaseDates) . ' FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
		'@phan-var array{customerID:int,customOrderNo:int,pricesNet:int,calcVat:int,shippingCost:string,shippingNet:int,shippingVat:string,customFields:string} $orderData';

		if(empty($orderData)){
			return we_html_tools::getHtmlTop('', $jscmd->getCmds(), '<body class="weEditorBody" onunload="doUnload()">
<table style="width:300px">
	<tr>
		<td colspan="2" class="defaultfont">' . we_html_tools::dialogLayout("<span class='defaultfont'>" . g_l('modules_shop', '[orderDoesNotExist]') . '</span>', g_l('modules_shop', '[loscht]')) . '</td>
	</tr>
</table></body>');
		}

		if($showDateFields){
			$orderData = array_merge($orderData, DB_WE::$inst->getAllFirstq('SELECT `type`,DATE_FORMAT(`date`,"' . g_l('date', '[format][mysql]') . '") FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid . ' AND `type` IN ("' . implode('","', $showDateFields) . '")'));
		}

		// get all needed information for order-data
		$_REQUEST['cid'] = $orderData['customerID'];
		//have all dates in $advanced
		if($showAdvanced){
			$advanced = DB_WE::$inst->getAllFirstq('SELECT odt.type,DATE_FORMAT(odt.date,"' . g_l('date', '[format][mysql]') . '") FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_DATES_TABLE . ' odt ON odt.ID=o.ID WHERE odt.type IN ("' . implode('","', $showAdvanced) . '") AND o.ID=' . $bid, false);
		} else {
			$advanced = [];
		}
		foreach($showBaseFields as $field){
			$advanced[$field] = $orderData[$field];
		}
		// prices are net?
		$pricesAreNet = $orderData['pricesNet'];
		// must calculate vat?
		$calcVat = (bool) $orderData['calcVat'];
		$customCartFields = $orderData['customFields'] ? we_unserialize($orderData['customFields']) : [];

		// ********************************************************************************
		// "Building the order infos"
		//

		$orderTable = new we_html_table(['class' => "default defaultfont shopOrderTable"], 0, ($calcVat ? 7 : 6));
		$orderTable->addHeaderRow([
			g_l('modules_shop', '[anzahl]'),
			g_l('modules_shop', '[Titel]'),
			g_l('modules_shop', '[Beschreibung]'),
			g_l('modules_shop', '[Preis]'),
			g_l('modules_shop', '[Gesamt]'),
			g_l('modules_shop', '[mwst]'),
			''
			], ['class' => "defaultfont lowContrast"]);

		$this->db->query('SELECT
	oi.orderDocID,
	oi.quantity,
	oi.Price,
	oi.customFields,
	IF(o.pricesNet||o.calcVat||IFNULL(oi.Vat,' . $mwst . ')=0,oi.Price, (oi.Price/(1+(IFNULL(oi.Vat,' . $mwst . ')/100))) )AS NetPrice,
	IFNULL(oi.Vat,' . $mwst . ') AS Vat,
	od.DocID,
	od.title,
	od.description,
	od.variant
FROM
	' . SHOP_ORDER_TABLE . ' o JOIN
	' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID JOIN
	' . SHOP_ORDER_DOCUMENT_TABLE . ' od ON oi.orderDocID=od.ID
WHERE o.ID=' . $bid);

		// loop through all articles
		$totalPrice = 0;
		$articleVatArray = [];
		while($this->db->next_record()){
			// all information for article
			//$ArticleId = $this->db->f('DocID'); // id of article (object or document) in shopping cart
			$tblOrdersId = (int) $this->db->f('orderDocID');
			// now determine VAT
			$articleVat = (float) $this->db->f('Vat');
			$articleVatInt = (int) ($articleVat * 100); //db only allows precision 2
			//
			// determine taxes - correct price, etc.
			$articlePrice = floatval($this->db->f('NetPrice')) * floatval($this->db->f('quantity'));
			$totalPrice += $articlePrice;

			// calculate individual vat for each article
			if($calcVat && $articleVat){
				if(!isset($articleVatArray[$articleVatInt])){ // avoid notices
					$articleVatArray[$articleVatInt] = 0;
				}
				$articleVatArray[$articleVatInt] += $articlePrice * ($articleVat / 100);
			}
// table row of one article
			$this->getOrderTableArticleRow($orderTable, $bid, $tblOrdersId, $waehr, $calcVat);
			// if this article has custom fields or is a variant - we show them in a extra rows
			// add variant.
			if($this->db->f('variant')){
				$rowid = $orderTable->addRows();
				$orderTable
					->setColAttributes($rowid, 0, ['colspan' => '4'])
					->setCol($rowid, 5, g_l('modules_shop', '[variant]') . ': ' . $this->db->f('variant'), ['class' => 'small', 'colspan' => '6']);
			}
		}
		// "Sum of order"
		// add shipping to costs
		// just calculate netPrice, gros, and taxes
		$shippingVat = (float) $orderData['shippingVat'];
		if(!isset($articleVatArray[(int) ($shippingVat * 100)])){
			$articleVatArray[(int) ($shippingVat * 100)] = 0;
		}

		if($orderData['shippingNet']){ // all correct here
			$shippingCostsNet = (float) $orderData['shippingCost'];
			$shippingCostsVat = $shippingCostsNet * $shippingVat / 100;
			$shippingCostsGros = $shippingCostsNet + $shippingCostsVat;
		} else {
			$shippingCostsGros = (float) $orderData['shippingCost'];
			$shippingCostsVat = $shippingCostsGros / ($shippingVat + 100) * $shippingVat;
			$shippingCostsNet = $shippingCostsGros / ($shippingVat + 100) * 100;
		}
		$articleVatArray[(int) ($shippingVat * 100)] += $shippingCostsVat;



		// add custom fields
		self::getOrderTableCustomFields($orderTable, $this->db->f('customFields'));
		$rowid = $orderTable->addRows(1);
		$orderTable->setCol($rowid, 6, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_new_article');"));
		$rowid = $orderTable->addRows(1, ['class' => 'preline']);
		$orderTable
			->setCol($rowid, 1, g_l('modules_shop', '[Preis]'), ['colspan' => '2'])
			->setCol($rowid, 3, we_base_util::formatNumber($totalPrice) . $waehr, ['colspan' => '2', 'class' => 'shopContentfontR bold']);

		if($calcVat){ // add Vat to price
			$totalPriceAndVat = $totalPrice;

			if($pricesAreNet){ // prices are net
				$totalPriceAndVat += $shippingCostsNet;
				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, g_l('modules_shop', '[shipping][shipping_package]') . ':', ['colspan' => '2'])
					->setCol($rowid, 3, '<a href="javascript:we_cmd(\'edit_shipping_cost\');">' . we_base_util::formatNumber($shippingCostsNet) . $waehr . '</a>', ['colspan' => '2',
						'class' => "shopContentfontR bold"]);

				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[plusVat]') . '</label>:', ['colspan' => '2'])
					//$orderTable->setCol($rowid, 3, ['colspan' => '2']);
					->setCol($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=0\';" type="checkbox" name="calculateVat" value="1" checked="checked" />');
				foreach($articleVatArray as $vatRate => $sum){
					if($vatRate){
						$totalPriceAndVat += $sum;

						$rowid = $orderTable->addRows();
						$orderTable
							->setCol($rowid, 1, $vatRate . '%', ['colspan' => '2'])
							->setCol($rowid, 3, we_base_util::formatNumber($sum) . $waehr, ['colspan' => '2', 'class' => 'shopContentfontR']);
					}
				}
				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, g_l('modules_shop', '[gesamtpreis]') . ':', ['colspan' => '2'])
					->setCol($rowid, 3, we_base_util::formatNumber($totalPriceAndVat) . $waehr, ['colspan' => '2', 'class' => "shopContentfontR bold"]);
			} else { // prices are gros
				$totalPrice += $shippingCostsGros;
				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, g_l('modules_shop', '[shipping][shipping_package]') . ':', ['colspan' => '2'])
					->setCol($rowid, 3, '<a href="javascript:we_cmd(\'edit_shipping_cost\');">' . we_base_util::formatNumber($shippingCostsGros) . $waehr . '</a>', [
						'colspan' => '2', 'class' => "shopContentfontR"])
					->setCol($rowid, 5, '(' . we_base_util::formatNumber($shippingVat) . '%)', ['class' => "shopContentfontR small"]);

				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, g_l('modules_shop', '[gesamtpreis]') . ':', ['colspan' => '2'])
					->setCol($rowid, 3, we_base_util::formatNumber($totalPrice) . $waehr, ['colspan' => '2', 'class' => "shopContentfontR bold"]);

				$rowid = $orderTable->addRows(1, ['class' => 'preline']);
				$orderTable
					->setCol($rowid, 1, '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[includedVat]') . '</label>:', ['colspan' => '2'])
					//$orderTable->setCol($rowid, 3, ['colspan' => '2']);
					->setCol($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=0\';" type="checkbox" name="calculateVat" value="1" checked="checked" />');
				foreach($articleVatArray as $vatRate => $sum){
					if($vatRate){
						$rowid = $orderTable->addRows();
						$orderTable
							->setCol($rowid, 1, $vatRate . ' %:', ['colspan' => '2'])
							->setCol($rowid, 3, we_base_util::formatNumber($sum) . $waehr, ['colspan' => '2', 'class' => 'shopContentfontR']);
					}
				}
			}
		} else {
			$totalPrice += $shippingCostsNet;
			$rowid = $orderTable->addRows(1, ['class' => 'preline']);
			$orderTable
				->setCol($rowid, 1, g_l('modules_shop', '[shipping][shipping_package]') . ':', ['colspan' => '2'])
				->setCol($rowid, 3, '<a href="javascript:we_cmd(\'edit_shipping_cost\')">' . we_base_util::formatNumber($shippingCostsNet) . $waehr . '</a>', ['colspan' => '2',
					'class' => 'shopContentfontR']);

			$rowid = $orderTable->addRows(1, ['class' => 'preline']);
			$orderTable
				->setCol($rowid, 1, '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[edit_order][calculate_vat]') . '</label>', ['colspan' => '2'])
				//$orderTable->setCol($rowid, 3, ['colspan' => '2']);
				->setCol($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=1\';" type="checkbox" name="calculateVat" value="1" />');

			$rowid = $orderTable->addRows(1, ['class' => 'preline']);
			$orderTable
				->setCol($rowid, 1, g_l('modules_shop', '[gesamtpreis]') . ':', ['colspan' => '2'])
				->setCol($rowid, 3, we_base_util::formatNumber($totalPrice) . $waehr, ['colspan' => '2', 'class' => "shopContentfontR bold"]);
		}

		//
		// "Sum of order"
		// ********************************************************************************
		// ********************************************************************************
		// "Additional fields in shopping basket"
		//
		// at last add custom shopping fields to order
		// table with orders ends here
		//
		// "Additional fields in shopping basket"
		// ********************************************************************************
		//
		// "Building the order infos"
		// ********************************************************************************
		// ********************************************************************************
		// "Html output for order with articles"
		//
		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $this->getOrderDataTable($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid, $customer, $fields)))
			->add(new we_html_multiIconBoxContentLine('', $orderTable->getHtml()))
			->add(new we_html_multiIconBoxContentLine('', self::getCustomCartFieldsTable($bid, $customCartFields)));

		return we_html_tools::getHtmlTop('', $jscmd->getCmds() .
				we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
						'bid' => $bid,
						'cid' => $orderData['customerID']
				])]), we_html_element::body(['class' => "weEditorBody", 'onload' => "hot = true", 'onunload' => "doUnload()"], (new we_html_multiIconBox($parts))->marginLeft()->get()));
	}

	private function getOrderDataTable(we_shop_statusMails $weShopStatusMails, array $showDateFields, array $advanced, array $orderData, int $bid, array $customer, array $fields): string{
		return '<table class="default defaultfont shopOrderDataTable withSpace">' .
			self::getMailDates($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid) . '
' . self::getCustomerFieldTable($customer, $fields ? $fields['orderCustomerFields'] : []) . '
			<tr><td colspan="9"><a href="javascript:we_cmd(\'edit_order_customer\');">' . g_l('modules_shop', '[order][edit_order_customer]') . '</a></td></tr>
			<tr><td colspan="9">' . (we_base_permission::hasPerm([we_base_perm::CUSTOMER_EDIT]) ? '<a href="javascript:we_cmd(\'we_customer_edit\');">' . g_l('modules_shop', '[order][open_customer]') . '</a>' : '') . ' </td></tr>
		</table>';
	}

	private function getOrderTableArticleRow(we_html_table $orderTable, int $bid, int $tblOrdersId, string $waehr, bool $calcVat): void{
		$Quantity = floatval($this->db->f('quantity'));
		$Price = floatval($this->db->f('NetPrice'));
		$articlePrice = $Quantity * $Price;
		$articleVat = (float) $this->db->f('Vat');
		$rowid = $orderTable->addRows();
		$colid = 0;
		$orderTable
			->setCol($rowid, $colid++, "<a href=\"javascript:var anzahl=prompt('" . g_l('modules_shop', '[jsanz]') . "','" . $Quantity . "'); if(anzahl != null){if(anzahl.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&anzahl='+anzahl;}}\">" . $Quantity . "</a>", [
				'class' => "shopContentfontR"])
			->setCol($rowid, $colid++, self::cutText($this->db->f('title'), 35))
			->setCol($rowid, $colid++, self::cutText($this->db->f('description'), 35))
			->setCol($rowid, $colid++, "<a href=\"javascript:var preis = window.prompt('" . g_l('modules_shop', '[jsbetrag]') . "','" . $Price . "'); if(preis != null ){if(preis.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&preis=' + preis; } }\">" . we_base_util::formatNumber($Price) . "</a>" . $waehr, [
				'class' => 'shopContentfontR'])
			->setCol($rowid, $colid++, we_base_util::formatNumber($articlePrice) . $waehr, ['class' => "shopContentfontR"]);
		if($calcVat){
			$orderTable->setCol($rowid, $colid++, '(' . "<a href=\"javascript:var vat = window.prompt('" . g_l('modules_shop', '[vat][new_vat_name]') . "','" . $articleVat . "'); if(vat != null ){if(vat.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&vat=' + vat; } }\">" . we_base_util::formatNumber($articleVat) . "</a>" . '%)', [
				'class' => "shopContentfontR small"]);
		}
		$orderTable->setCol($rowid, $colid++, we_html_button::create_button(we_html_button::TRASH, "javascript:check=window.confirm('" . g_l('modules_shop', '[jsloeschen]') . "'); if (check){document.location.href=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&bid=" . $bid . "&deleteaarticle=" . $tblOrdersId . "';}", '', "", !we_base_permission::hasPerm([
					we_base_perm::SHOP_DELETE_ARTICLE])), ['class' => 'buttonCol']);
	}

	private static function getOrderTableCustomFields(we_html_table $orderTable, string $cf): void{
		if(!$cf){
			return;
		}
		$cf = we_unserialize($cf);
		$caField = '';
		foreach($cf as $key => $value){
			$caField .= $key . ': ' . $value . '; ';
		}
		$rowid = $orderTable->addRows();
		$orderTable
			->setColAttributes($rowid, 0, ['colspan' => '4'])
			->setCol($rowid, 5, $caField, ['class' => 'small', 'colspan' => '6']);
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		$bid = we_base_request::int('bid');
		switch(we_base_request::string('cmd', '')){
			case 'add_article':
				if(we_base_request::float('anzahl') == 0){
					$jscmd->addMsg(g_l('modules_shop', '[keinezahl]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				$tmp = explode('_', we_base_request::string('add_article', ''));
				$isObj = ($tmp[1] == we_shop_shop::OBJECT);
				$id = intval($tmp[0]);

				// check for variant or customfields
				$customFieldsTmp = [];
				$customField = we_base_request::string('we_customField');
				if($customField){
					$fields = explode(';', trim($customField));
					if(is_array($fields)){
						foreach($fields as $field){
							$fieldData = explode('=', $field);
							if(is_array($fieldData) && count($fieldData) == 2){
								$customFieldsTmp[trim($fieldData[0])] = trim($fieldData[1]);
							}
						}
					}
				}

				$variant = we_base_request::string(we_base_variants::REQUEST);
				$serialDoc = we_shop_Basket::getserial($id, ($isObj ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT), $variant, $customFieldsTmp);

				// shop vats must be calculated
				$orderArray = getHash('SELECT customerData,priceName FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
				$customer = we_unserialize($orderArray['customerData']);

				try{
					$standardVat = we_shop_vats::getStandardShopVat();
				} catch (Throwable $e){
					$standardVat = null;
				}

				if(we_shop_category::isCategoryMode()){
					$wedocCategory = $serialDoc[we_listview_base::CATEGORY_FIELD];
					$stateField = we_shop_vatRule::getStateField();
					$billingCountry = !empty($customer[$stateField]) ? (string) $customer[$stateField] : we_shop_category::getDefaultCountry();
					$catId = !empty($serialDoc[we_shop_shop::CATEGORY_FIELD_NAME]) ? $serialDoc[we_shop_shop::CATEGORY_FIELD_NAME] : 0;

					try{
						$shopVat = we_shop_category::getShopVatByIdAndCountry($catId, $wedocCategory, $billingCountry)->vat;
					} catch (Throwable $e){
						$shopVat = ($standardVat ?: null);
					}
				} else {
					$shopVat = (isset($serialDoc[we_shop_shop::VAT_FIELD_NAME]) ?
						we_shop_vats::getShopVATById($serialDoc[we_shop_shop::VAT_FIELD_NAME])->vat :
						0);
					$catId = 0;
				}

				$type = ($isObj ? 'object' : 'document');
				$pub = $serialDoc['WE_Published'];

				$orderDocID = fI('SELECT ID FROM ' . SHOP_ORDER_DOCUMENT_TABLE . ' WHERE DocID=' . $id . ' AND type="' . $type . '" AND variant="' . $this->db->escape($variant) . '" AND Published=' . (is_numeric($pub) ? 'FROM_UNIXTIME(' . $pub . ')' : $pub));
				if(!$orderDocID){
					$data = $serialDoc;
					unset($data['we_shoptitle'], $data['we_shopdescription'], $data[we_shop_shop::ARTICLE_CUSTOM_FIELD], $data['shopvat'], $data[we_shop_shop::CATEGORY_FIELD_NAME], $data[we_listview_base::VARIANT_FIELD]);
					//add document first
					$this->db->query('INSERT INTO ' . SHOP_ORDER_DOCUMENT_TABLE . ' SET ' . we_database_base::arraySetter([
							'DocID' => $id,
							'type' => $type,
							'variant' => $variant,
							'Published' => is_numeric($pub) ? sql_function('FROM_UNIXTIME(' . $pub . ')') : $pub,
							'title' => strip_tags($serialDoc['shoptitle']),
							'description' => strip_tags($serialDoc['shopdescription']),
							'CategoryID' => intval($catId),
							'SerializedData' => we_serialize($data, SERIALIZE_JSON, false, 5, true)
					]));
					$orderDocID = $this->db->getInsertId();
				}

				//need pricefield:
				$pricename = ($orderArray['priceName'] ?: 'price');
				// now insert article to order:
				$this->db->query('INSERT INTO ' . SHOP_ORDER_ITEM_TABLE . ' SET ' .
					we_database_base::arraySetter(([
						'orderID' => $bid,
						'orderDocID' => $orderDocID,
						'quantity' => we_base_request::float('anzahl', 0),
						'Price' => $serialDoc[$pricename],
						'Vat' => ($shopVat ? $shopVat->vat : 0),
						'customFields' => $serialDoc[we_shop_shop::ARTICLE_CUSTOM_FIELD] ? we_serialize($serialDoc[we_shop_shop::ARTICLE_CUSTOM_FIELD], SERIALIZE_JSON, false, 0, true) : null,
				])));

				break;

			case 'payVat':
				$calcVat = we_base_request::int('pay', 0);

				// update all orders with this orderId
				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET calcVat=' . $calcVat . ' WHERE ID=' . $bid)){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_calculateVat_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_calculateVat_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;

			case 'delete_shop_cart_custom_field':
				$cartfield = we_base_request::string('cartfieldname');
				if($cartfield){
					$customFields = we_unserialize(fS('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid));
					unset($customFields[$cartfield]);

					// update all orders with this orderId
					if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
								'customFields' => $customFields ? we_serialize($customFields, SERIALIZE_JSON, false, 0, true) : null,
							]) . ' WHERE ID=' . $bid)
					){
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field_success]'), $cartfield), we_base_util::WE_MESSAGE_NOTICE);
						return;
					}
					$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field_error]'), $cartfield), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				break;

			case 'save_shop_cart_custom_field':
				$cartfield = we_base_request::string('cartfieldname');
				if($cartfield){
					$customFields = we_unserialize(fS('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid));
					$customFields[$cartfield] = we_base_request::string('cartfieldvalue', '');

					// update all orders with this orderId
					if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
								'customFields' => we_serialize($customFields, SERIALIZE_JSON, false, 0, true),
							]) . ' WHERE ID=' . $bid)
					){
						$jscmd->addCmd('doClickShopOrder', $bid);
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_saved_cart_field_success]'), $cartfield), we_base_util::WE_MESSAGE_NOTICE);
					} else {
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_saved_cart_field_error]'), $cartfield), we_base_util::WE_MESSAGE_ERROR);
					}
				} else {
					$jscmd->addMsg(g_l('modules_shop', '[field_empty_js_alert]'), we_base_util::WE_MESSAGE_ERROR);
				}
				$jscmd->addCmd('close_dialog');
				echo we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js') . $jscmd->getCmds(), we_html_element::body());
				exit;

			case 'save_shipping_cost':
				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
							'shippingCost' => we_base_request::float('weShipping_costs'),
							'shippingNet' => we_base_request::int('weShipping_isNet'),
							'shippingVat' => we_base_request::float('weShipping_vatRate'),
						]) . ' WHERE ID=' . $bid)
				){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_shipping_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_shipping_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;

			case 'save_order_customer':
				$customer = we_base_request::stringList('weCustomerOrder');
				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
							'customerData' => we_serialize($customer, SERIALIZE_JSON, false, 5, true),
						]) . ' WHERE ID=' . $bid)){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_customer_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_customer_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;
		}
	}

	function processVariables(): void{
		$page = we_base_request::int('page', -1);
		if($page >= 0){
			$this->page = $page;
		}
	}

	//some functions from edit_shop_properties

	private static function cutText(string $val, int $length = 0): string{
		return $length && strlen($val) > $length ?
			'<span class="cutText" title="' . $val . '" style="max-width: ' . $length . 'em;">' . $val . '</span>' :
			$val;
	}

	private function getOrderCustomerData(int $orderId, array $felder = []): array{
		// get Customer
		$customerDb = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=(SELECT customerID FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId . ')', $this->db, MYSQLI_ASSOC);

		$customerOrder = we_unserialize(fS('SELECT customerData FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId));

		//used only if edit customer data is selected!!!
		//only data from order - return all fields, fill in unknown fields from customer-db
		// default values are fields saved with order
		$ret = array_merge($customerDb, $customerOrder);
		return ($felder ? //return only selected fields
			array_intersect_key($ret, array_flip($felder)) :
			$ret);
	}

	public function getHomeScreen(): string{

		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = $feldnamen[$i] ?? '';
		}
		$fe = explode(',', $feldnamen[3]);
		$classid = $fe[0];

		$resultO = array_shift($fe);

		$resultD = fB('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');


		$content = we_html_button::create_button('pref_shop', "javascript:we_cmd('settings');", '', "", !we_base_permission::hasPerm([we_base_perm::USER_NEW])) . '<br/>' .
			we_html_button::create_button('payment_val', "javascript:we_cmd('payment_val');", '', "", !we_base_permission::hasPerm([we_base_perm::USER_NEW])) . '<br/>';
		if(($resultD) && $resultO){ //docs and objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop' . "&pnt=editor&top=1&typ=document '") . '<br/>';
		} elseif((!$resultD) && $resultO){ // no docs but objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop' . "&pnt=editor&top=1&typ=object&ViewClass=$classid '") . '<br/>';
		} elseif(($resultD) && !$resultO){ // docs but no objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop' . "&pnt=editor&top=1&typ=document '") . '<br/>';
		}

		return parent::getActualHomeScreen(we_modules_info::SHOP, $content);
	}

	private static function getCustomCartFieldsTable(int $bid, array $customCartFields): string{
		$customCartFieldsTable = new we_html_table(['class' => "default shopCustomFields"], count($customCartFields) + 1, 4);
		$customCartFieldsTable->addHeaderRow([g_l('modules_shop', '[order_comments]')]);
		$rowid = 1;
		foreach($customCartFields as $key => $value){
			$customCartFieldsTable
				->setCol($rowid, 0, $key, ['class' => 'bold'])
				->setCol($rowid, 1, nl2br((string) $value))
				->setCol($rowid, 2, we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('edit_shop_cart_custom_field','" . $key . "');"))
				->setCol($rowid, 3, we_html_button::create_button(we_html_button::TRASH, "javascript:check=window.confirm('" . sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field]'), $key) . "'); if (check) { document.location.href=WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edbody&we_cmd[0]=delete_shop_cart_custom_field&bid=" . $bid . "&cartfieldname=" . $key . "'; }"));
			$rowid++;
		}
		$customCartFieldsTable->setCol($rowid, 0, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('edit_shop_cart_custom_field');"));
		return $customCartFieldsTable->getHtml();
	}

	private static function getCustomerFieldTable(array $customer, array $customerFields): string{
		$customerFieldTable = // first show fields Forename and surname
			(empty($customer['Forename']) ? '' : '
		<tr class="preline">
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . g_l('modules_customer', '[Forname]') . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top" colspan="6">' . $customer['Forename'] . '</td>
		</tr>') .
			(empty($customer['Surname']) ? '' : '
		<tr>
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . g_l('modules_customer', '[Surname]') . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top;" colspan="6">' . $customer['Surname'] . '</td>
		</tr>');

		$customer = array_intersect_key($customer, array_flip($customerFields));
		unset($customer['Surname'], $customer['Forename']);
		foreach($customer as $key => $value){
			/* switch($key){
			  case $this->CLFields['stateField']:
			  if($this->CLFields['stateFieldIsISO']){
			  $value = g_l('countries', '[' . strtoupper($value) . ']');
			  }
			  break;
			  case $this->CLFields['languageField']:
			  if($this->CLFields['languageFieldIsISO']){
			  $value = g_l('languages', '[' . strtolower($value) . ']');
			  }
			  break;
			  } */
			$customerFieldTable .= '
		<tr>
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . $key . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top;" colspan="6">' . $value . '</td>
		</tr>';
		}
		return $customerFieldTable;
	}

	private static function getMailDates(we_shop_statusMails $weShopStatusMails, array $showDateFields, array $advanced, array $orderData, int $bid): string{
		$dateform = '00.00.0000';

		$ret = '<tr>
<td>' . g_l('modules_shop', '[bestellnr]') . '</td><td class="shopContentfontR bold" style="padding-right:2em;">' . ($orderData['customOrderNo'] ?: $bid) . '</td>
</tr>';
		foreach($showDateFields as $f){
			$field = (string) $f;
			$ret .= '<tr>
				<td style="width:98px; padding-right:2em;">' . $weShopStatusMails->FieldsText[$field] . '</td>
				<td class="shopContentfontR" style="padding-right:2em;">
					<div id="div_Calendar_' . $field . '">' . ((empty($advanced[$field]) || $advanced[$field] === $dateform) ? '-' : $advanced[$field]) . '</div>
					<input type="hidden" name="' . $field . '" id="hidden_Calendar_' . $field . '" value="' . (empty($advanced[$field]) || ($advanced[$field] == $dateform) ? '-' : $advanced[$field]) . '" />
				</td>
				<td>' .
				//we_html_button::create_button(we_html_button::CALENDAR, "javascript:$(this).datepicker('dialog', '" . (empty($advanced[$field]) || ($advanced[$field] == $dateform) ? '' : $advanced[$field]) . "', function(val,ob){CalendarChanged('" . $field . "',ob.lastVal,val);},{},event); ", '', '', false, true, '', false, '', 'button_Calendar_' . $field) .
				'</td>' . $weShopStatusMails->getEMailHandlerCode('', '') . '
				</tr>';
		}
		return $ret;
	}

}
