<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: aacab26fe68a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 10:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_a extends we_weTag_tag{
	private const removeAttribs = [
		'id',
		'shop',
		'amount',
		'delshop',
		'delarticle',
		'hidedirindex',
		'shopname',
		'return',
		'edit',
		'type',
		'button',
		'hrefonly',
		'confirm',
		'editself',
		'delete',
		'params'
	];

	/** @var bool */
	private static $isListview = false;

	/** @var bool */
	private static $isSelf = false;

	public static function parse(array $attribs, string $content): string{
		return '<?php ob_start();?>' . $content . '<?= ' . we_weTag_parser::printTag('a', $attribs, 'ob_get_clean()', false, true) . '; ?>';
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::$doc = we_base_WE::getWeDoc();
		self::$isListview = !empty(we_tag_listview::$current);
		if(self::$isListview && we_tag_listview::$current instanceof we_listview_pseudo){
			$id = (int) we_tag_listview::$current->id;
			self::$isSelf = false;
		} else {
			// check for id attribute
			if(($foo = self::attributeMissing($attribs, ['id' => false], __CLASS__))){
				return $foo;
			}
			self::$isSelf = self::getAttribute('id', $attribs, 0, we_base_request::STRING) === 'self';
			// get attributes
			$id = (int) (self::$isSelf ?
				(!defined('WE_REDIRECTED_SEO') ?
				we_base_WE::$mainDoc['id'] :
				0) :
				self::getAttribute('id', $attribs, 0, we_base_request::INT));
		}
		$confirm = self::getAttribute('confirm', $attribs, '', we_base_request::STRING);
		$button = self::getAttribute('button', $attribs, false, we_base_request::BOOL);
		$hrefonly = self::getAttribute('hrefonly', $attribs, false, we_base_request::BOOL);
		$return = self::getAttribute('return', $attribs, false, we_base_request::BOOL);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$shop = self::getAttribute('shop', $attribs, false, we_base_request::BOOL);
		$delarticle = self::getAttribute('delarticle', $attribs, false, we_base_request::BOOL);
		$delshop = self::getAttribute('delshop', $attribs, false, we_base_request::BOOL);
		list($anchor, $param) = we_base_util::getParamStr(self::getAttribute('params', $attribs, '', we_base_request::STRING));

		$edit = self::getAttribute('edit', $attribs, '', we_base_request::STRING) ?: ($shop || $delarticle || $delshop ? 'shop' : '');
		$url = self::getUrl($id, $attribs);
		if(!$url && !(we_base_WE::$mainDoc['doc'] instanceof we_contents_template)){
			return (we_base_WE::$editmode ? self::parseError('in we:a attribute id not exists!') : '');
		}

		if($edit){
			$param = array_merge($param, self::getEditParams($edit, $attribs, $delshop, $delarticle));
		}

		if($return){
			$param['we_returnpage'] = we_http_util::getScriptName() . '?' . $_SERVER['QUERY_STRING'];
		}

		if($hrefonly){
			return $url . $anchor . ($param ? '?' . http_build_query($param) : '');
		}

		//	remove unneeded attributes from array
		$tagAttribs = self::removeAttribs($attribs, self::removeAttribs);

		if($button){ //	show button
			// FIXME: inline js
			$tagAttribs['type'] = 'button';
			$tagAttribs['onclick'] = ($target ? "var we_win=window.open('','" . $target . "');we_win" : 'self') .
				".document.location='" . $url . $anchor . ($param ? '?' . http_build_query($param) : '') . "';";

			$tagAttribs = self::removeAttribs($tagAttribs, ['target']); //	not html - valid

			if($confirm){
				$confirm = str_replace("'", "\\'", $confirm);
				$tagAttribs['onclick'] = 'if(window.confirm(\'' . $confirm . '\')){' . $tagAttribs['onclick'] . '}';
			}
			return self::htmlTag('button', $tagAttribs, $content, true);
		}
//	show normal link
		$tagAttribs['href'] = $url . $anchor . ($param ? '?' . http_build_query($param) : '');

		if($confirm){
			$tagAttribs['onclick'] = 'return window.confirm(\'' . $confirm . '\'));';
		}

		return self::htmlTag('a', $tagAttribs, $content, true);
	}

	private static function getUrl(int $id, array $attribs): string{
		if(self::$isSelf && defined('WE_REDIRECTED_SEO')){
			return WE_REDIRECTED_SEO;
		}
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		// init variables
		$row = getHash('SELECT Path,IsFolder,IsDynamic FROM ' . FILE_TABLE . ' WHERE ID=' . $id);
		$url = (!$row ? '' : $row['Path'] . ($row['IsFolder'] ? '/' : ''));
		$path_parts = pathinfo($url);
		return ($hidedirindex && TAGLINKS_DIRECTORYINDEX_HIDE && seoIndexHide($path_parts['basename']) ?
			($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
			$url);
	}

	private static function getEditParams(string $edit, array $attribs, bool $delshop, bool $delarticle): array{
		$delete = self::getAttribute('delete', $attribs, false, we_base_request::BOOL);
		$editself = self::getAttribute('editself', $attribs, false, we_base_request::BOOL);

		switch($edit){
			case 'shop':
				if($delshop && (($foo = self::attributeMissing($attribs, ['shopname' => false], __CLASS__)))){
					throw new we_exception_parser($foo);
				}

				$amount = self::getAttribute('amount', $attribs, 1, we_base_request::INT);
				$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);
				$ifShopname = ($shopname ? ['shopname' => $shopname] : []);
				if($delshop){ //delete basket
					return array_merge(['deleteshop' => 1, 't' => time()], $ifShopname);
				}
				$customReq = [];
				if(self::$isListview){
					list('id' => $articleID, 'type' => $type, 'req' => $customReq) = we_tag_listview::$current->getShopArticleID();
				} else {
					try{
						$articleID = we_base_WE::getWeObj()->ID;
						$type = we_shop_shop::OBJECT;
					} catch (Exception $ex){
						$articleID = self::$doc->ID;
						$type = we_shop_shop::DOCUMENT;
					}
				}

				// is it a variant ????
				$variant = // normal variant on document
					(self::$doc->Variant ? // normal listView or document
					[we_base_variants::REQUEST => self::$doc->Variant] :
					// variant inside shoplistview!
					(self::$isListview && we_tag_listview::$current->f(we_listview_base::VARIANT_FIELD) ?
					[we_base_variants::REQUEST => we_tag_listview::$current->f(we_listview_base::VARIANT_FIELD)] :
					[]));

				$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction');

				//	preview mode in seem
				if($we_transaction){
					$doc = we_contents_root::getSessionDocument($we_transaction);
					if($doc instanceof we_contents_objectFile){
						$type = we_shop_shop::OBJECT;
					}
				}

				return ($delarticle ? // delete article
					array_merge(['del_shop_artikelid' => $articleID, 'type' => $type, 't' => time()], $variant, $customReq, $ifShopname) :
					// increase/decrease amount of articles
					array_merge(['shop_artikelid' => $articleID, 'shop_anzahl' => $amount, 'type' => $type, 't' => time()], $variant, $customReq, $ifShopname)
					);

			case 'object':
				$obj = we_base_WE::$we_obj;
				'@phan-var we_contents_objectFile $obj';

				$oid = (self::$isListview ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) :
					($obj && $editself ? $obj->ID : 0));

				//FIXME: make sure only the selected object can be deleted - sth unique not user-known has to be added to prevent denial of service
				return ($oid ?
					[($delete ? we_tag_delete::DEL_OBJ_PARAM : we_tag_write::EDIT_OBJ_PARAM) => $oid] :
					['edit_object' => 1]);

			case 'document':
				$did = (self::$isListview ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) :
					($editself ? self::$doc->ID : 0));

				//FIXME: make sure only the selected document can be deleted - sth unique not user-known has to be added to prevent denial of service
				return ($did ?
					[($delete ? we_tag_delete::DEL_DOC_PARAM : we_tag_write::EDIT_DOC_PARAM) => $did] :
					['edit_document' => 1]
					);
		}
		return [];
	}

}
