<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_css extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(($foo = self::attributeMissing($attribs, ['id' => false], __CLASS__))){
			return $foo;
		}
		$src = id2path(($id = self::getAttribute('id', $attribs, 0, we_base_request::INT)));

		if(!$src){
			return '';
		}
		self::$doc = we_base_WE::getWeDoc();

		$nolink = false;
		switch(self::getAttribute('applyto', $attribs, defined('CSSAPPLYTO_DEFAULT') ? CSSAPPLYTO_DEFAULT : 'around', we_base_request::STRING)){
			case 'around' :
				break;
			case 'wysiwyg' :
				$nolink = true;
			case 'all' :
				switch(self::getAttribute('media', $attribs, '', we_base_request::STRING)){
					case '':
					case 'screen':
					case 'all':
						// we still need addDocumentCss() because JS getDocumentCss() won't apply it when applyTo=wysiwyg!
						self::$doc->addDocumentCss($id);
						break;
				}
				break;
		}
		//	remove not needed elements
		$attribs = self::removeAttribs($attribs, ['id', 'applyto']);
		$rel = self::getAttribute('rel', $attribs, 'stylesheet', we_base_request::STRING);
		if($rel === 'preload'){
			$attribs['as'] = 'style';
			header('Link: <' . $src . '>; rel=preload; as=style', false);
		}
		if(we_base_WE::$editmode){
			// these stylesheets are applied to wysiwyg using addDocumentCss depending on attribute "applyto":
			// so prevent JS getDocumentCss() from applying it again or when applyto=around!
			$src .= '?wysiwyg=0';
		}
		$attribs['rel'] = $rel;
		$attribs['type'] = 'text/css';
		$attribs['href'] = $src;


		return $nolink ? '' : self::htmlTag('link', $attribs) . "\n";
	}

}
