<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 23e425ca457e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-21 16:58 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_delete extends we_weTag_tag{
	const DEL_DOC_PARAM = 'we_delDocument_ID';
	const DEL_OBJ_PARAM = 'we_delObject_ID';

	/** @var array<string,bool> */
	public static $deletedTypes = [];

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$id = (int) self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);
		$userid = self::getAttribute('userid', $attribs, 0, we_base_request::STRING); // deprecated  use protected=true instead
		$protected = self::getAttribute('protected', $attribs, false, we_base_request::BOOL);
		$admin = self::getAttribute('admin', $attribs, '', we_base_request::STRING);
		$mail = self::getAttribute('mail', $attribs, '', we_base_request::EMAIL);
		$mailfrom = self::getAttribute('mailfrom', $attribs, '', we_base_request::EMAIL);
		$doctype = self::getAttribute('doctype', $attribs, '', we_base_request::STRING);
		$classid = self::getAttribute('classid', $attribs, '', we_base_request::INT);
		$pid = self::getAttribute('pid', $attribs, 0, we_base_request::INT);
		$forceedit = self::getAttribute('forceedit', $attribs, false, we_base_request::BOOL);
		$mailtext = $subject = '';

		switch($type){
			case 'document':
				$docID = $id ?: we_base_request::int(self::DEL_DOC_PARAM);
				if(!$docID){
					return '';
				}
				$doc = new we_document_webEdition($docID);
				$table = FILE_TABLE;
				if($doctype){
					$doctypeID = fI('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE Text LIKE "' . DB_WE::$inst->escape($doctype) . '" AND IsFolder=0');
					if($doc->DocType != $doctypeID){
						self::$deletedTypes[$type] = false;
						return '';
					}
				}
				if($mail){
					$mailtext = sprintf(g_l('global', '[std_mailtext_delDoc]'), $doc->Path);
					$subject = g_l('global', '[std_subject_delDoc]');
				}
				break;
			case 'object':
				$docID = $id ?: we_base_request::int(self::DEL_OBJ_PARAM, $id);
				if(!$docID){
					return '';
				}
				$doc = new we_contents_objectFile($docID);
				$table = OBJECT_FILES_TABLE;
				if($classid && $doc->TableID != $classid){//FIXME: isClassFolder()
					self::$deletedTypes[$type] = false;
					return '';
				}
				if($mail){
					$mailtext = sprintf(g_l('global', '[std_mailtext_delObj]'), $doc->Path);
					$subject = g_l('global', '[std_subject_delObj]');
				}
				break;
			default:
				return '';
		}

		if($pid && $doc->ParentID != $pid){
			self::$deletedTypes[$type] = false;
			return '';
		}

		$isOwner = !empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser']['ID']) && (
			($protected && $_SESSION['webuser']['ID'] == $doc->WebUserID) ||
			($userid && $_SESSION['webuser']['ID'] == $doc->getElementI($userid))
			);


		$isAdmin = !empty($_SESSION['webuser']['registered']) && $admin && !empty($_SESSION['webuser'][$admin]);

		if($isAdmin || $isOwner || $forceedit){
			we_base_delete::deleteEntry($docID, $table);
			self::$deletedTypes[$type] = true;
			if($mail && ($to = we_mail_address::parse($mail))){
				$from = $mailfrom ? we_mail_address::parse($mailfrom) : new we_mail_address('dontReply@' . $_SERVER['SERVER_NAME']);
				$phpmail = new we_mail_mail([$to], $subject, $from);
				$phpmail->addTextPart(trim($mailtext));
				$phpmail->buildMessage();
				$phpmail->send();
			}
		} else {
			self::$deletedTypes[$type] = false;
		}
		return '';
	}

}
