<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifFieldEmpty extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(empty($attribs['match']) && ($foo = self::attributeMissing($attribs, ['name' => false], __CLASS__))){
			echo $foo;
			return false;
		}
		self::$doc = we_base_WE::getWeDoc();

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$orig_name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING) ?: self::getAttribute('match', $attribs, '', we_base_request::STRING);

		if(!we_tag_listview::$current){
			echo self::parseError(g_l('parser', '[field_not_in_lv]'), __CLASS__);
			return true;
		}

		$name = (we_tag_listview::$current->f($orig_name) ? $orig_name : self::getPostName($orig_name));

		switch($type){
			case 'calendar' :
				$cal = we_tag_listview::$current->getCalendarStruct();
				if(!empty($cal)){
					if($cal['date'] < 0 || empty($cal['storage'])){
						return true;
					}
					$ed = $sd = 0;
					switch($orig_name){
						case 'day':
							$sd = mktime(0, 0, 0, $cal['month_human'], $cal['day_human'], $cal['year_human']);
							$ed = mktime(23, 59, 59, $cal['month_human'], $cal['day_human'], $cal['year_human']);
							break;
						case 'month':
							$sd = mktime(0, 0, 0, $cal['month_human'], 1, $cal['year_human']);
							$ed = mktime(23, 59, 59, $cal['month_human'], $cal['numofentries'], $cal['year_human']);
							break;
						case 'year':
							$sd = mktime(0, 0, 0, 1, 1, $cal['year_human']);
							$ed = mktime(23, 59, 59, 12, 31, $cal['year_human']);
							break;
					}
					if($sd && $ed){
						foreach($cal['storage'] as $entry){
							if($sd < $entry && $ed > $entry){
								return false;
							}
						}
					}
				}
				return true;
			case 'multiobject':
				$data = we_unserialize(we_tag_listview::$current ?
					we_tag_listview::$current->f($orig_name) :
					self::$doc->getElementS($orig_name));
				$objects = array_filter($data['objects'] ?? $data);
				return empty($objects);
			case 'object' : //Bug 3837: erstmal die Klasse rausfinden um auf den Eintrag object_X zu kommen
				if(we_tag_listview::$current instanceof we_listview_document){ // listview/document with objects included using we:object
					return empty(we_tag_listview::$current->f($name));
				}
				$name = strpos($orig_name, '/') === false ? $orig_name : substr(strrchr($orig_name, '/') ?: '', 1);
				$objectid = fI('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($name) . '"');

				return empty(we_tag_listview::$current->f(we_listview_base::OBJECT_FIELD_PREFIX . $objectid));
			case 'checkbox':
			case 'binary':
			case 'img':
			case 'flashmovie':
			case 'collection':
			case 'video':
				return empty(we_tag_listview::$current->f($name));
			case 'float':
				return floatval(we_tag_listview::$current->f($name)) === floatval(0);
			case 'int':
				return intval(we_tag_listview::$current->f($name)) === 0;
			case 'href' :
				if(we_tag_listview::$current instanceof we_listview_objectBase){
					if(!($hrefArr = we_tag_listview::$current->f($name) ? we_unserialize(we_tag_listview::$current->f($name)) : [])){
						return true;
					}
					$hreftmp = trim(we_document_base::getHrefByArray($hrefArr));
					/**
					 * TODO: file_exists only work when the choosen webEdition document is in the correct DOCUMENT_ROOT
					 * but within multi domains we have cross DOCUMENT_ROOT references!!!
					 */
					if(!$hreftmp || $hreftmp === '/' || ($hreftmp[0] === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp)))){
						return true;
					}
					return false;
				}

				// we must check $match . we_base_link::MAGIC_INT_LINK for block-Postfix instead of $match (which exists only for href type = ext): #6422
				$isInBlock = ( we_tag_listview::$current->f($orig_name . we_base_link::MAGIC_INT_LINK) || we_tag_listview::$current->f($orig_name) ) ? false : true;
				$name = $isInBlock ? self::getPostName($orig_name) : $orig_name;

				$int = (we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK) == '') ? 0 : we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK);
				if($int){ // for type = href int
					$intID = (int) we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK_ID);
					return !($intID && id2path($intID));
				}
				$hreftmp = we_tag_listview::$current->f($name);
				if($hreftmp[0] === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp))){
					return true;
				}

				break; //see return of function
			default:
				$tmp = we_unserialize(we_tag_listview::$current->f($name), '', true);
				if(is_array($tmp)){
					return empty($tmp);
				}
			//no break;
		}
		return empty(we_tag_listview::$current->f($name));
	}

}
