<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: d5bcf73330c9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-28 19:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifShopCategory extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		$field = self::getAttribute('field', $attribs, 'id', we_base_request::STRING);
		$match = intval(self::getAttribute('match', $attribs, false, we_base_request::INT));
		$ignorefallbacks = self::getAttribute('ignorefallbacks', $attribs, false, we_base_request::BOOL);
		$lvOnly = self::getAttribute('lvOnly', $attribs, false, we_base_request::BOOL);
		self::$doc = we_base_WE::getWeDoc();

		if(we_tag_listview::$current){
			$catID = (int) we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME) ?: 0;
			$f = we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD);
			$wedocCategory = $f ? we_base_request::filterIntList($f) : [];
		} elseif(!$lvOnly){
			$catID = self::$doc->getElementI(we_shop_shop::CATEGORY_FIELD_NAME);
			$wedocCategory = self::$doc->Category ?: [];
		} else {
			return false;
		}

		$validArr = we_shop_category::checkGetValidID($catID, $wedocCategory, true);
		$validID = intval($validArr['id']);

		switch($field){
			case 'is_from doc_object':
				return ($validArr['state'] === we_shop_category::IS_CAT_FALLBACK_TO_WEDOCCAT) || ($validArr['state'] === we_shop_category::IS_CAT_FALLBACK_TO_WEDOCCAT_AND_ACTIVE);
			case 'is_fallback_to_standard':
				return ($validArr['state'] === we_shop_category::IS_CAT_FALLBACK_TO_STANDARD);
			case 'is_fallback_to_active':
				return ($validArr['state'] === we_shop_category::IS_CAT_FALLBACK_TO_ACTIVE) || ($validArr['state'] === we_shop_category::IS_CAT_FALLBACK_TO_WEDOCCAT_AND_ACTIVE);
			case 'is_destinationprinciple':
				try{
					we_shop_category::getShopCatFieldByID($validID, $wedocCategory, 'DestPrinciple');
					return true;
				} catch (we_exception_shop $ex){
					return false;
				}
			case 'id':
				if(!$match){
					return $ignorefallbacks ? boolval($catID) : boolval($validID);
				}
				return $ignorefallbacks ? $catID === $match : $validID === $match;

			default:
				return false;
		}
	}

}
