<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 2282ee756c0c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-10 17:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_path extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(($current = we_tag_listview::$current) && $current instanceof we_listview_pseudo){//listdir
			return $current->Path;
		}

		$field = self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$dirfield = self::getAttribute('dirfield', $attribs, $field, we_base_request::STRING);
		$index = self::getAttribute('index', $attribs, '', we_base_request::STRING);
		$oldHtmlspecialchars = self::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL);
		$fieldforfolder = self::getAttribute('fieldforfolder', $attribs, false, we_base_request::BOOL);
		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		$sep = self::getAttribute('separator', $attribs, '/', we_base_request::RAW_CHECKED);
		$home = self::getAttribute('home', $attribs, 'home', we_base_request::RAW_CHECKED);
		$hidehome = self::getAttribute('hidehome', $attribs, false, we_base_request::BOOL);
		$class = self::getAttribute('class', $attribs, '', we_base_request::STRING);
		$style = self::getAttribute('style', $attribs, '', we_base_request::STRING);
		$max = self::getAttribute('max', $attribs, 0, we_base_request::INT);

		$doc = self::getDocForTag($docAttr, true);
		$pID = $doc->ParentID;
		$realPID = 0;

		$indexArray = $index ? explode(',', $index) : ['index.html', 'index.htm', 'index.php', 'default.htm', 'default.html', 'default.php'];

		$class = $class ? ' class="' . $class . '"' : '';
		$style = $style ? ' style="' . $style . '"' : '';

		$path = (!in_array(($text = $doc->getText()), $indexArray, true) ?
				$sep . ($oldHtmlspecialchars ? htmlspecialchars($doc->getElementS($field, $text, true)) : ($doc->getElementS($field, $text, true))) :
				'');

		$q = '"' . implode('","', array_map('escape_sql_query', $indexArray)) . '"';
		$mdf = md5($dirfield);
		$mf = md5($field);

		while($pID){
			list('ID' => $fileID, 'Path' => $filePath, 'Text' => $fileName, 'Dat' => $show) = (getHash('SELECT
	f.ID,f.Path,f.Text,c.Dat
FROM ' .
							FILE_TABLE . ' f LEFT JOIN ' .
							CONTENT_TABLE . ' c ON (c.DID=f.ID AND c.DocumentTable="tblFile" AND c.nHash=x\'' . $mdf . '\')
WHERE
	f.ParentID=' . intval($pID) . ' AND
	f.IsFolder=0 AND
	f.Text IN (' . $q . ') AND
	f.IsPublished=1 LIMIT 1') ?:
					[0, '', '', '']
					);

			if($fileID){
				if(!$show && $fieldforfolder){
					$show = fS('SELECT	c.Dat FROM ' . CONTENT_TABLE . ' c
WHERE
	c.DocumentTable="tblFile" AND
	c.DID=' . intval($fileID) . ' AND
	c.nHash=x\'' . $mf . '\'');
				}
				$show = $show ?: $fileName;
				$link_pre = ($fileID != $doc->ID ? '<a href="' . $filePath . '"' . $class . $style . '>' : '');
			} else {
				$link_pre = '';
				//set show empty, we get it when we query the parent
				$show = '';
			}

			//for multidomains we stop if we find a "document root"
			list('pid' => $pID, 'ParentID' => $realPID, 'Text' => $folderName) = getHash('SELECT IF(urlMap,0,ParentID) AS pid,ParentID,Text FROM ' . FILE_TABLE . ' WHERE ID=' . intval($pID)) ?:
					['pid' => 0, 'ParentID' => 0, 'Text' => ''];

			//if no name is given, take the folder name
			$show = $show ?: $folderName;

			$cutted = ($max ? self::cutText($show, $max) : $show);

			$path = (!$pID && $hidehome ? '' : $sep) .
					$link_pre .
					($oldHtmlspecialchars ? htmlspecialchars($cutted) : $cutted) .
					($link_pre ? '</a>' : '') . $path;
		}

		if($hidehome){
			return $path;
		}

		list('ID' => $fileID, 'Path' => $filePath, 'Dat' => $show) = getHash('SELECT
	f.ID,
	f.Path,
	c.Dat
FROM ' .
						FILE_TABLE . ' f LEFT JOIN ' .
						CONTENT_TABLE . ' c ON (c.DID=f.ID AND c.DocumentTable="tblFile" AND c.nHash=x\'' . $mf . '\')
WHERE
	f.ParentID=' . $realPID . ' AND
	f.IsFolder=0 AND
	f.Text IN (' . $q . ') AND
	f.IsPublished=1 LIMIT 1') ?: ['ID' => 0, 'Path' => '', 'Dat' => ''];

		$text = (string) ($show ?: $home); //@phan-suppress-current-line PhanImpossibleCondition
		$link_pre = ($fileID ? '<a href="' . $filePath . '"' . $class . $style . '>' : ''); //@phan-suppress-current-line PhanImpossibleCondition

		$cutted = ($max ? self::cutText($text, $max) : $text);

		return $link_pre . ($oldHtmlspecialchars ? htmlspecialchars($cutted) : $cutted) . ($link_pre ? '</a>' : '') . $path;
	}

}
