<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 4d6b7c42b65f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-16 20:16 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_shopVat extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::$doc = we_base_WE::getWeDoc();
		if(!we_shop_category::isCategoryMode()){
			// in webEdition - EditMode
			$allVats = we_shop_vats::getAllShopVATs();
			$values = [];

			foreach($allVats as $shopVat){
				$values[$shopVat->id] = $shopVat->vat . ' - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
			}

			$attribs['name'] = we_shop_shop::VAT_FIELD_NAME;
			try{
				$weShopVat = self::$doc->issetElement(we_shop_shop::VAT_FIELD_NAME) ? we_shop_vats::getShopVATById((int) self::$doc->getElementS(we_shop_shop::VAT_FIELD_NAME)) : we_shop_vats::getStandardShopVat();
			} catch (we_exception_shop $e){
				$weShopVat = we_shop_vats::getStandardShopVat();
			}
			// use a defined name for this
			if(we_base_WE::$editmode){
				$fieldname = 'we_' . self::$doc->Name . '_attrib[' . we_shop_shop::VAT_FIELD_NAME . ']';
				return we_html_element::span(['class' => 'we we_shopVat'], we_html_tools::select($fieldname, $values, 1, $weShopVat->id));
			}

			return (string) $weShopVat->vat;
		}

		$field = self::getAttribute('field', $attribs, 'vat', we_base_request::STRING);
		$id = self::getAttribute('id', $attribs, false, we_base_request::INT);
		$shopcategoryid = self::getAttribute('shopcategoryid', $attribs, false, we_base_request::INT);
		$wedoccategories = self::getAttribute('wedoccategories', $attribs, '', we_base_request::STRING);
		$customerid = self::getAttribute('customerid', $attribs, false, we_base_request::INT);
		$countrycode = self::getAttribute('countrycode', $attribs, false, we_base_request::STRING);
		$iso = $field === 'countrycode';

		$fieldMap = [
			'id' => 'id',
			'vat' => 'vat',
			'name' => 'text',
			'country' => 'territory',
			'countrycode' => 'territory',
			'is_standard' => 'standard',
			'is_fallback_to_standard' => 'is_vat_fallback_to_standard',
			'is_fallback_to_prefs' => 'is_vat_fallback_to_prefs',
			'is_country_fallback_to_prefs' => 'is_country_fallback_to_prefs'
		];
		$field = $fieldMap[$field] ?? 'vat';

		$vat = null;
		if($id){
			try{
				$vat = we_shop_vats::getShopVATById($id);
			} catch (we_exception_shop $e){

			}
		}
		if(!$vat){
			$countryArr = we_shop_category::getFullCountryFromCustomer(true, [], $customerid);
			$countryArr['country'] = $countrycode ?: $countryArr['country']; //country delivered as attribute has highest priority

			if($field === 'is_country_fallback_to_prefs'){
				return $countryArr['country'] && $countryArr['isFallback'] ? '1' : '0';
			}

			$weShopVatRule = we_shop_vatRule::getShopVatRule();
			if(!$weShopVatRule->executeVatRule(empty($countryArr['customer']) ? [] : $countryArr['customer'], $countryArr['country'])){
				return $field === 'vat' ? '0' : '';
			}
			$catId = $shopcategoryid ?: (self::$doc->getElementI(we_shop_shop::CATEGORY_FIELD_NAME));
			$wedocCategory = $wedoccategories ?: self::$doc->Category;
			try{
				$vat = we_shop_category::getShopVatByIdAndCountry($catId, $wedocCategory, $countryArr['country']/* , ($field === 'is_vat_fallback_to_standard'), ($field === 'is_vat_fallback_to_prefs') */);
			} catch (Throwable $e){
				$vat = null;
			}
		}

		if($vat){
			switch($field){
				case 'is_vat_fallback_to_standard':
				case 'is_vat_fallback_to_prefs':
					return '0';
				case 'standard':
					return $vat->standard ? '1' : '0';
				case 'text':
					return $id ? $vat->text : $vat->getNaturalizedText();
				case 'territory':
					if(!$iso){
						$lang = (string) (array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');
						//TODO: get translation from we_shop_vat
						return we_base_country::getTranslation($vat->$field, we_base_country::LANGUAGE, $lang) ?: $vat->territory;
					}
					return $vat->territory;
				default:
					return (string) $vat->$field;
			}
		}

		return '0';
	}

}
