<?php
declare(strict_types=1);
/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
define('TEMPORARY_DOC_TABLE', TBL_PREFIX . 'tblTemporaryDoc');

abstract class we_update_updater{

	static function replayUpdateDB(string $specFile = ''): void{
		//FIXME: even in update, only execute queries on enabled modules
		include_once(WEBEDITION_PATH . 'liveUpdate/includes/define.inc.php');
		include_once(WEBEDITION_PATH . 'liveUpdate/classes/liveUpdateFunctions.class.php');
		$lf = new liveUpdateFunctions();
		we_base_errorHandler::setHandling('sql', false);
		if($specFile){
			$lf->executeQueriesInFiles(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'sqldumps/' . $specFile);
		} else {
			$inactive = we_modules_info::getAllInactiveModules();
			$allRemoveTables = [];
			foreach($inactive as $cur){//add new tables
				$tables = we_modules_info::getModuleField((string) $cur, 'tables');
				if($tables){
					$allRemoveTables = array_merge($allRemoveTables, $tables);
				}
			}
			$d = dir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'sqldumps');
			while(false !== ($entry = $d->read())){
				if(substr($entry, -4) === '.sql'){
					$tabName = addTblPrefix(basename($entry, '.sql'));
					if(in_array($tabName, $allRemoveTables)){
						if(DB_WE::$inst->isTabExist($tabName) && fI('SELECT COUNT(1) FROM ' . $tabName) === 0){
							DB_WE::$inst->delTable($tabName);
						}
					} else {
						$lf->executeQueriesInFiles(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'sqldumps/' . $entry);
					}
				}
			}
			$d->close();
		}
		we_base_errorHandler::setHandling('sql', true);
		if(($entries = $lf->getQueryLog('error'))){
			t_e('Errors while updating tables', $entries);
		}
	}

	//added in Barrhorn
	public static function updateObjects(we_database_base $db){
		$where = "(DefaultValues LIKE '{\"WorkspaceFlag\":%' OR DefaultValues LIKE '%\"WorkspaceFlag\":0,%' OR DefaultValues LIKE '%\"WorkspaceFlag\":\"0\",%' OR DefaultValues LIKE '%\"WorkspaceFlag\":1,%' OR DefaultValues LIKE '%\"WorkspaceFlag\":\"1\",%')";
		$entries = $db->getAllFirstq('SELECT ID,DefaultValues,Workspaces,DefaultWorkspaces,Templates FROM ' . OBJECT_TABLE . ' WHERE ' . $where, true, MYSQLI_ASSOC);
		'@phan-var array<int,array{DefaultValues:string,Workspaces:string,DefaultWorkspaces:string,Templates:string}> $entries';

		foreach($entries as $id => $entry){
			$data = we_unserialize((string) $entry['DefaultValues']);
			//$oldData = we_unserialize((string) $entry['DefaultValues']);
			$defaultValues = [];

			$defaultValues['WE_CSS_FOR_CLASS'] = array_filter(we_base_request::filterIntList($data['WE_CSS_FOR_CLASS']));
			$defaultValues['WorkspaceFlag'] = !is_bool($data['WorkspaceFlag']) ? we_base_request::convertBool((string) $data['WorkspaceFlag']) : $data['WorkspaceFlag'];

			foreach($data as $fieldName => &$oldattribs){
				'@phan-var string $fieldName';
				if(!in_array(($type = explode('_', $fieldName)[0]), array_merge(array_keys(we_contents_objectBase::DEFAULTS_VALUES), ['meta']))){
					continue;
				}

				if($type === 'meta'){
					$fieldName = 'select' . substr($fieldName, 4);
					$type = we_contents_objectBase::TYPE_SELECT;
				}

				$defaultValues[$fieldName] = [];
				$typeDefaults = array_merge(we_contents_objectBase::COMMON_DEFAULTS_VALUES,
					(we_contents_objectBase::DEFAULTS_VALUES[$type]['defaults'] ?? [])
				);
				$typeDefField = we_contents_objectBase::DEFAULTS_VALUES[$type]['defField'];

				// rename default to new defaults names
				if(isset($oldattribs['default'])){
					$oldattribs[$typeDefField] = $oldattribs['default'];
					unset($oldattribs['default']);
				}

				// rename or convert some attribs
				switch($type){
					case we_contents_objectBase::TYPE_DATE:
						$oldattribs[$typeDefField] = (isset($oldattribs[$typeDefField]) && $oldattribs[$typeDefField] === true);
						if(isset($oldattribs['defaultThumb'])){
							$oldattribs[$typeDefField] = !is_bool($oldattribs['defaultThumb']) ? we_base_request::convertBool((string) $oldattribs['defaultThumb']) : $oldattribs['defaultThumb'];
						}
						break;
					case we_contents_objectBase::TYPE_HREF:
						if(!empty($oldattribs[$typeDefField])){
							$tmp = is_array($oldattribs[$typeDefField]) ? $oldattribs[$typeDefField] : we_unserialize($oldattribs[$typeDefField]);
							$oldattribs[$typeDefField] = array_filter([
								'int' => isset($tmp['int']) ? (!is_bool($tmp['int']) ? we_base_request::convertBool((string) $tmp['int']) : $tmp['int']) : true,
								'intID' => isset($tmp['intID']) ? intval($tmp['intID']) : 0,
								'extPath' => isset($tmp['extPath']) ? filter_var((string) $tmp['extPath'], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES) : '',
							]);
						}
						break;
					case we_contents_objectBase::TYPE_LINK:
						if(!empty($oldattribs[$typeDefField])){
							$oldattribs[$typeDefField] = is_array($oldattribs[$typeDefField]) ? $oldattribs[$typeDefField] : we_unserialize($oldattribs[$typeDefField]);
						}
						break;
					case we_contents_objectBase::TYPE_SELECT:
						if(!empty($oldattribs['meta'])){
							$oldattribs['selectKeys'] = array_keys($oldattribs['meta']);
							$oldattribs['selectValues'] = array_values($oldattribs['meta']);
						}
						break;
					case we_contents_objectBase::TYPE_MULTIOBJECT:
						$oldattribs[$typeDefField] = isset($oldattribs['meta']) ? (is_array($oldattribs['meta']) ? $oldattribs['meta'] : we_unserialize($oldattribs['meta'])) : ($oldattribs[$typeDefField] ?? []);
						$oldattribs['tableid'] = $oldattribs['tableid'] ?? ($oldattribs['class'] ?? 0);
						break;
					case we_contents_objectBase::TYPE_TEXT:
						$oldattribs['wysiwyg'] = !empty($oldattribs['dhtmledit']) || !empty($oldattribs['wysiwyg']);
						break;
				}

				// convert and write attribs to new defaultsArray if they belong to the given fieldType and differ from respective defaults
				foreach($typeDefaults as $key => $attrib){
					if(isset($oldattribs[$key])){
						list($attribDefault, $attribType) = $attrib;
						switch($attribType){
							case we_base_request::BOOL:
								$oldattribs[$key] = !is_bool(($oldattribs[$key])) ? we_base_request::convertBool((string) $oldattribs[$key]) : $oldattribs[$key];
								break;
							case we_base_request::INT:
								$oldattribs[$key] = (int) $oldattribs[$key];
								break;
							case we_base_request::FLOAT:
								$oldattribs[$key] = (float) ((string) $oldattribs[$key] === '' ? 0 : we_base_util::std_numberformat((string) $oldattribs[$key]));
								break;
							case we_base_request::HTML:
								$oldattribs[$key] = strtr((string) $oldattribs[$key], ['<?' => '<!--?', '?>' => '?-->', '<script' => '<!--script', '</script>' => '</script-->']);
								break;
							case we_base_request::STRING:
								$oldattribs[$key] = filter_var((string) $oldattribs[$key], FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
								break;
							case we_base_request::INTLISTA:
								if($attrib[2] ?? false){ // do not apply array_filter
									$oldattribs[$key] = (!is_array($oldattribs[$key]) ? we_unserialize((string) $oldattribs[$key]) : $oldattribs[$key]);
									$tmp = [];
									foreach($oldattribs[$key] as $v){
										$tmp[] = intval($v);
									}
									$oldattribs[$key] = $tmp;
									break;
								}
								$oldattribs[$key] = we_base_request::filterIntList($oldattribs[$key], []);
								break;
							case we_base_request::STRING_LIST:
								if($attrib[2] ?? false){ // do not apply array_filter
									$tmp = [];
									foreach($oldattribs[$key] as $v){
										$tmp[] = filter_var((string) $v, FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);
									}
									$oldattribs[$key] = $tmp;
									break;
								}
								$oldattribs[$key] = we_base_request::filterStringList($oldattribs[$key], []);
								break;
							case 'arrayHref':
							case 'arrayLink':
								// these cases are already processed
								break;
						}
						if($oldattribs[$key] !== $attribDefault){
							$defaultValues[$fieldName][$key] = $oldattribs[$key];
						}
					}
				}
			}

			//t_e('new defaults', $id, $defaultValues, $oldData);

			$db->query('UPDATE ' . OBJECT_TABLE . ' SET ' . we_database_base::arraySetter([
					'DefaultValues' => we_serialize($defaultValues, SERIALIZE_JSON),
					'Workspaces' => trim($entry['Workspaces'], ','),
					'DefaultWorkspaces' => trim($entry['DefaultWorkspaces'], ','),
					'Templates' => trim($entry['Templates'], ',')
				]) . ' WHERE ID=' . $id);
		}
	}

	private static function updateObjectFilesX(we_database_base $db = null, array $progress = []): array{
		$db = $db ?: new DB_WE();

		$init = $progress = ($progress ?: [
			'pos' => 0,
			'maxID' => 0,
			'max' => fI('SELECT COUNT(1) FROM ' . OBJECT_TABLE)
		]);

		$maxStep = 15;

		if(!$progress['max'] || ($progress['pos'] > $progress['max'])){//finished
			$db->delCol(OBJECT_TABLE, 'strOrder');
			return [];
		}

		//change old tables to have different prim key
		$tables = $db->getAllq('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE ID>' . intval($init['maxID']) . ' ORDER BY ID LIMIT ' . $maxStep, true);
		foreach($tables as $table){
			if(!$db->isColExist(OBJECT_X_TABLE . $table, 'OF_DocumentTable')){
				$db->addCol(OBJECT_X_TABLE . $table, 'OF_DocumentTable', "enum('tblObjectFiles','tblTemporaryDoc') NOT NULL DEFAULT 'tblObjectFiles'", 'OF_ID');
				$db->query('ALTER TABLE ' . $db->escape(OBJECT_X_TABLE . $table) . ' DROP PRIMARY KEY, ADD PRIMARY KEY (OF_DocumentTable,OF_ID)');
			}
		}

		$progress['maxID'] = end($tables) ?: $progress['maxID'];
		//make sure we always progress
		$progress['pos'] += (count($tables) ?: 1);

		return array_merge($progress, ['text' => 'Classes ' . $progress['pos'] . ' / ' . $progress['max']]);
	}

	//added in 7.1
	private static function updateObjectFiles2X(we_database_base $db): void{
		$tables = $db->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true);
		foreach($tables as $table){
			$entries = $db->getAllq('SHOW COLUMNS FROM ' . OBJECT_X_TABLE . $table . ' WHERE Field LIKE "' . we_contents_objectFile::TYPE_MULTIOBJECT . '_%"');
			'@phan-var array{Field:string,Type:string,Null:string,Key:string,Default:string|null,Extra:string}[] $entries';
			foreach($entries as $entry){
				$field = $entry['Field'];
				$data = $db->getAllFirstq('SELECT OF_ID,`' . $field . '` FROM ' . OBJECT_X_TABLE . $table . ' WHERE `' . $field . '` REGEXP "^a:[0-9]+:{.*}$"', false);
				foreach($data as $id => $cur){
					'@phan-var string $cur';
					$cur = we_unserialize($cur);
					$db->query('UPDATE ' . OBJECT_X_TABLE . $table . ' SET ' . we_database_base::arraySetter([
							$field => implode(',', $cur['objects']),
						]) . ' WHERE OF_ID=' . $id);
				}
			}
			//added in Barrhorn
			foreach($db->getAllq('SHOW COLUMNS FROM ' . OBJECT_X_TABLE . $table . ' WHERE Field LIKE "meta_%"') as $col){
				'@phan-var array{Field:string,Type:string,Null:string,Key:string,Default:string|null,Extra:string} $col';
				$field = (string) $col['Field'];
				$db->renameCol(OBJECT_X_TABLE . $table, $field, preg_replace('|^meta_(.*)|', we_contents_objectBase::TYPE_SELECT . '_$1', $field));
			}
		}
	}

	public static function fixInconsistentTables(we_database_base $db = null): void{//from backup
		$db = $db ?: DB_WE::$inst;

		if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			//delete old schedule entries, they can do more harm
			if($db->isColExist(SCHEDULE_TABLE, 'Schedpro')){
				//$db->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE Schedpro REGEXP \'"task":"[0-9]"\'');
			}
			//FIXME: this has to be integrated in we_delete code!
			$db->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE documentTable="tblFile" AND DID NOT IN (SELECT ID FROM ' . FILE_TABLE . ')');

			$db->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE documentTable="tblObjectFiles" AND DID NOT IN (SELECT ID FROM ' . OBJECT_FILES_TABLE . ')');
		}
		//clean customerfilter
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$db->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="tblFile" AND modelId NOT IN (SELECT ID FROM ' . FILE_TABLE . ')');
			$db->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="tblObjectFiles" AND modelId NOT IN (SELECT ID FROM ' . OBJECT_FILES_TABLE . ')');
			$db->query('DELETE FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="tblnavigation" AND modelId NOT IN (SELECT ID FROM ' . NAVIGATION_TABLE . ')');
		}

		$db->query('DELETE FROM ' . CAPTCHADEF_TABLE . ' WHERE ID NOT IN (SELECT ID FROM ' . TEMPLATES_TABLE . ')');


		$tables = $db->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true);
		foreach($tables as $table){
			if($db->isTabExist(OBJECT_X_TABLE . $table)){
				$db->query('DELETE FROM ' . OBJECT_X_TABLE . $table . ' WHERE OF_ID NOT IN (SELECT ID FROM ' . OBJECT_FILES_TABLE . ')');
			}
		}
		if($db->isColExist(SESSION_TABLE, 'lockid')){
			$db->delCol(SESSION_TABLE, 'lockid');
		}
		$db->query('UPDATE ' . OBJECT_FILES_TABLE . ' of JOIN ' . OBJECT_TABLE . ' o ON o.Path=SUBSTRING_INDEX(of.Path,"/",2) SET of.TableID=o.ID WHERE of.TableID=0');
	}

	public static function meassure(string $name): void{
		static $last = 0;
		static $times = [];
		$last = $last ?: microtime(true);
		if($name == '-1'){
			t_e('notice', 'time for updates', $times);
			return;
		}
		$now = microtime(true);
		$times[] = $name . ': ' . round($now - $last, 3);
		$last = $now;
	}

	public static function removeObsoleteFiles(string $path = ''): bool{
		$path = $path ?: WEBEDITION_PATH . 'liveUpdate/includes/';
		if(!is_file($path . 'del.files')){
			return true;
		}

		$delFiles = [];
		if(($all = file($path . 'del.files', FILE_IGNORE_NEW_LINES))){
			foreach($all as $cur){
				$recursive = false;
				if($cur[0] === '!'){
					$cur = substr($cur, 1);
					$recursive = true;
				}
				if(file_exists(WEBEDITION_PATH . $cur)){
					if(is_file(WEBEDITION_PATH . $cur)){
						$delFiles[] = $cur;
						we_base_file::deleteLocalFile(WEBEDITION_PATH . $cur);
					} elseif(is_dir(WEBEDITION_PATH . $cur)){
						$delFiles[] = 'Folder: ' . $cur;
						we_base_file::deleteLocalFolder(WEBEDITION_PATH . $cur, $recursive);
					}
				}
			}
		}
		we_base_file::deleteLocalFile($path . 'del.files');
		file_put_contents($path . 'deleted.files', ($all ? "Deleted Files: " . count($delFiles) . "\n\n" . implode("\n", $delFiles) : "File del.files empty"));

		return true;
	}

	private static function updateLinkList(we_database_base $db, we_database_base $db2): void{
		$db->query('SELECT ID,DID,Name,Dat FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="tblFile" AND Type="linklist"');
		while($db->next_record()){
			$tmp = we_unserialize($db->f('Dat'));
			$name = $db->f('Name');
			$new = [];
			foreach($tmp as $i => $cur){
				$i = '_' . ($i + 1);
				$new[] = $i;
				$newName = 'linkblk_' . $name . '_' . $i;
				unset($cur['img_attribs'], $cur['jswin_attribs']);
				$db2->query('REPLACE INTO ' . CONTENT_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocumentTable' => 'tblFile',
						'Type' => 'block',
						'DID' => $db->f('DID'),
						'Name' => $newName,
						'Dat' => we_serialize(array_filter($cur))
				]));
			}
			$db2->query('UPDATE ' . CONTENT_TABLE . ' SET ' . we_database_base::arraySetter([
					'BDID' => null,
					'Dat' => we_serialize($new),
					'Type' => 'link'
				]) . ' WHERE ID=' . $db->f('ID'));
		}
	}

	/** //FIXME: remove in cardada
	 *
	 * @param we_database_base $db
	 * @param array $progress
	 * @return array
	 */
	private static function updateTempTable(we_database_base $db, array $progress = []): array{
		if(!$db->isTabExist(TEMPORARY_DOC_TABLE)){
			return [];
		}

		if(!$progress){
			$db->query('DELETE FROM `' . TEMPORARY_DOC_TABLE . '` WHERE DocTable="tblFile" AND DocumentID NOT IN (SELECT ID FROM `' . FILE_TABLE . '`)');
			$db->query('DELETE FROM `' . TEMPORARY_DOC_TABLE . '` WHERE DocTable="tblObjectFiles" AND DocumentID NOT IN (SELECT ID FROM `' . OBJECT_FILES_TABLE . '`)');
		}

		$progress = ($progress ?: [
			'pos' => 0,
			'max' => fI('SELECT COUNT(1) FROM ' . TEMPORARY_DOC_TABLE)
		]);
		$maxStep = 50;

		if(!$progress['max'] || ($progress['pos'] > $progress['max'])){//finished
			$db->delTable(TEMPORARY_DOC_TABLE);
			return [];
		}

		$all = $db->getAllq('SELECT td.DocumentID,td.DocTable,td.DocumentObject,of.TableID FROM `' . TEMPORARY_DOC_TABLE . '` td LEFT JOIN ' . OBJECT_FILES_TABLE . ' `of` ON td.DocumentID=of.ID ORDER BY td.DocTable="tblFile" DESC,of.TableID,td.DocumentID LIMIT ' . $maxStep);
		$docIds = $ofIds = [];
		foreach($all as $cur){
			$data = we_unserialize($cur['DocumentObject']);
			switch($cur['DocTable']){
				case 'tblFile':
					$doc = new we_document_html();
					$doc->we_updaterTransferTempDoc($data); //@phan-suppress-current-line PhanDeprecatedFunction
					$docIds[] = $cur['DocumentID'];
					break;
				case 'tblObjectFiles':
					$doc = new we_contents_objectFile();
					$doc->we_updaterTransferTempDoc($data); //@phan-suppress-current-line PhanDeprecatedFunction
					$ofIds[] = $cur['DocumentID'];
					break;
			}
		}
		if($docIds){
			$db->query('DELETE FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocTable="tblFile" AND DocumentID IN (' . implode(',', $docIds) . ')');
		}
		if($ofIds){
			$db->query('DELETE FROM ' . TEMPORARY_DOC_TABLE . ' WHERE DocTable="tblObjectFiles" AND DocumentID IN (' . implode(',', $ofIds) . ')');
		}
		$progress['pos'] = min($progress['pos'] + $maxStep, $progress['max'] + 1);

		return array_merge($progress, ['text' => 'Temptable ' . $progress['pos'] . ' / ' . $progress['max']]);
	}

	private static function updateVersionsTable(we_database_base $db): void{
		$db->query('UPDATE ' . VERSIONS_TABLE . ' SET binaryPath=REPLACE(binaryPath,"' . WE_VERSION_DIR . '","") WHERE binaryPath LIKE "' . WEBEDITION_DIR . '%"');
		$db->query('UPDATE ' . VERSIONS_TABLE . ' SET binaryPath=NULL WHERE binaryPath=""');

		//clean old binaries
		$dids = $db->getAllq('SELECT DISTINCT documentID FROM ' . VERSIONS_TABLE . ' WHERE documentTable="tblFile" AND binaryPath!="" AND ContentTypeIN("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '") ORDER BY documentID LIMIT 1000', true);
		foreach($dids as $did){
			$ret = true;
			foreach(glob(WE_VERSION_DIR . $did . '_*', GLOB_NOSORT) as $file){
				$ret = $ret && we_base_file::deleteLocalFile($file); //@phan-suppress-current-line PhanRedundantConditionInLoop
			}
			if($ret){
				$db->query('UPDATE ' . VERSIONS_TABLE . ' SET binaryPath=NULL WHERE documentTable="tblFile" AND documentID=' . $did);
			}
		}
	}

	private static function updateVersionsTable2(we_database_base $db, we_database_base $db2): void{
		if(!fB('SELECT 1 FROM ' . VERSIONS_TABLE . ' WHERE documentElements LIKE "a\\%3A%" OR documentScheduler LIKE "a\\%3A%" LIMIT 1')){
			return;
		}

		$db->query('SELECT ID,documentElements FROM ' . VERSIONS_TABLE . '  WHERE documentElements LIKE "a\\%3A%"');
		while($db->next_record()){
			$db2->query('UPDATE ' . VERSIONS_TABLE . ' SET ' . we_database_base::arraySetter([
					'documentElements' => sql_function('x\'' . bin2hex(we_serialize(array_filter(we_unserialize(html_entity_decode(urldecode($db->f('documentElements')), ENT_QUOTES))), SERIALIZE_JSON, false, 6)) . '\''),
				]) . ' WHERE ID=' . $db->f('ID'));
		}
		$db->query('SELECT ID,documentScheduler FROM ' . VERSIONS_TABLE . '  WHERE documentScheduler LIKE "a\\%3A%"');
		while($db->next_record()){
			$db2->query('UPDATE ' . VERSIONS_TABLE . ' SET ' . we_database_base::arraySetter([
					'documentScheduler' => sql_function('x\'' . bin2hex(we_serialize(array_filter(we_unserialize(html_entity_decode(urldecode($db->f('documentScheduler')), ENT_QUOTES))), SERIALIZE_JSON, false, 6)) . '\''),
				]) . ' WHERE ID=' . $db->f('ID'));
		}
	}

	private static function updateCustomer(we_database_base $db): void{
		$order = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="customer" AND pref_name="EditSort"');
		if(!$order){
			return;
		}
		$order = explode(',', $order);
		$meta = $db->metaFieldNames(CUSTOMER_TABLE);
		$defaultCols = ['ID', 'Password', 'Forename', 'Surname', 'LoginDenied', 'MemberSince_Date', 'LastLogin_Date', 'LastAccess_Date', 'AutoLogin', 'AutoLoginDenied',
			'Mod_Date',
			'ModifiedBy', 'Username',];
		$last = 'FIRST';
		foreach($defaultCols as $col){
			$db->moveCol(CUSTOMER_TABLE, $col, $last);
			$last = $col;
		}

		$newOrder = [];
		foreach($meta as $i => $col){
			if(!in_array($col, $defaultCols)){
				$pos = array_search($i, $order);
				$newOrder[$col] = $pos;
				$i++;
			}
		}
		asort($newOrder, SORT_NUMERIC);
		//now we have the defined order
		//sort by groups again
		$groups = [
			'' => []
		];
		foreach(array_keys($newOrder) as $col){
			'@phan-var string $col';
			$tmp = explode('_', $col, 2);
			if(empty($tmp[1])){
				array_unshift($tmp, '');
			}
			$groups[$tmp[0]][] = $tmp[1];
		}
		$last = (string) end($defaultCols);
		foreach($groups as $group => $entries){
			foreach($entries as $entry){
				$name = ($group === '' ? '' : $group . '_') . $entry;
				$db->moveCol(CUSTOMER_TABLE, $name, $last);
				$last = $name;
			}
		}

		$db->query('DELETE FROM ' . SETTINGS_TABLE . ' WHERE tool="customer" AND pref_name="EditSort"');
	}

	private static function updateCustomerFilters(we_database_base $db): void{
		if(!$db->isColExist(NAVIGATION_TABLE, 'CustomerFilter')){
			return;
		}

		$all = $db->getAllq('SELECT ID,CustomerFilter,WhiteList,BlackList,Customers FROM ' . NAVIGATION_TABLE . " WHERE CustomerFilter REGEXP '^a:[0-9]+:{.*}$'");
		'@phan-var array<int,array{ID:int,CustomerFilter:string,WhiteList:string,BlackList:string,Customers:string}> $all';
		foreach($all as $a){
			$db->query('UPDATE ' . NAVIGATION_TABLE . ' SET ' . we_database_base::arraySetter([
					'CustomerFilter' => we_serialize(we_unserialize($a['CustomerFilter']), SERIALIZE_JSON),
					'WhiteList' => trim($a['WhiteList'], ','),
					'BlackList' => trim($a['BlackList'], ','),
					'Customers' => trim($a['Customers'], ','),
				]) . ' WHERE ID=' . $a['ID']);
		}
		$allx = $db->getAllq('SELECT modelId,filter,whiteList,blackList,specificCustomers FROM ' . CUSTOMER_FILTER_TABLE . " WHERE filter REGEXP '^a:[0-9]+:{.*}$'");
		'@phan-var array<int,array{modelId:int,filter:string,whiteList:string,blackList:string,specificCustomers:string}> $allx';
		foreach($allx as $b){
			$db->query('UPDATE ' . CUSTOMER_FILTER_TABLE . ' SET ' . we_database_base::arraySetter([
					'filter' => we_serialize(we_unserialize($b['filter']), SERIALIZE_JSON),
					'whiteList' => trim($b['whiteList'], ','),
					'blackList' => trim($b['blackList'], ','),
					'specificCustomers' => trim($b['specificCustomers'], ','),
				]) . ' WHERE modelId=' . $b['modelId']);
		}
	}

	private static function updateSetting(): void{
		$data = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="weShippingControl" AND pref_value LIKE "%weShipping%"');
		if($data){
			$shippingControl = unserialize(strtr($data, [
				'O:17:"weShippingControl"' => 'O:' . strlen('we_shop_shippingControl') . ':"we_shop_shippingControl"',
				'O:10:"weShipping"' => 'O:' . strlen('we_shop_shipping') . ':"we_shop_shipping"',
				'O:17:"weshippingcontrol"' => 'O:' . strlen('we_shop_shippingControl') . ':"we_shop_shippingControl"',
				'O:10:"weshipping"' => 'O:' . strlen('we_shop_shipping') . ':"we_shop_shipping"'
			]));
			if(is_object($shippingControl)){
				$shippingControl->save();
			}
		}
	}

	private static function updateScheduler(we_database_base $db, we_database_base $db2): void{
		if(!$db->isColExist(SCHEDULE_TABLE, 'Schedpro')){
			return;
		}
		$db->query('SELECT * FROM ' . SCHEDULE_TABLE . '  WHERE Schedpro LIKE "%time%"');
		while($db->next_record()){
			$dat = $db->getRecord();
			$s = we_unserialize($dat['Schedpro']);
			$dat = array_merge($dat, [
				'months' => $dat['rerun'] === we_schedule_scheduler::RERUN_YEAR ? sql_function('b\'' . implode('', $s['months']) . '\'') : null,
				'days' => in_array($dat['rerun'], [we_schedule_scheduler::RERUN_YEAR, we_schedule_scheduler::RERUN_MONTH]) ? sql_function('b\'' . implode('', $s['days']) . '\'') : null,
				'weekdays' => $dat['rerun'] === we_schedule_scheduler::RERUN_WEEK ? sql_function('b\'' . implode('', $s['weekdays']) . '\'') : null,
				'time' => date('Y-m-d H:i', $s['time']),
				'ParentID' => $s['ParentID'] ?? null,
				'DoctypeID' => $s['DoctypeID'] ?? null,
				'DoctypeDefault' => $s['doctypeAll'] ?? 0,
				'CategoryIDs' => $s['CategoryIDs'] ?: null,
			]);

			unset($s['rerun'], $s['active'], $s['months'], $s['days'], $s['weekdays'], $s['type'], $s['time'], $s['task']);
			$dat['Schedpro'] = we_serialize(array_filter($s), SERIALIZE_JSON);

			$db2->query('REPLACE INTO ' . SCHEDULE_TABLE . ' SET ' . we_database_base::arraySetter($dat));
		}
		$db->delCol(SCHEDULE_TABLE, 'Schedpro');
	}

	private static function updateThumbnails(we_database_base $db): void{
		$thumbDir = we_contents_thumbnail::getThumbDirectory(true);
		we_base_file::checkAndMakeFolder($thumbDir);
		//create and move files
		foreach($db->getAllq('SELECT ID FROM ' . THUMBNAILS_TABLE, true) as $thumbID){
			we_base_file::checkAndMakeFolder($thumbDir . '/' . $thumbID);
			foreach(glob($thumbDir . '/*_' . $thumbID . '_*.*', GLOB_NOSORT) as $filename){
				$file = str_replace('_' . $thumbID . '_', '_', basename($filename));
				we_base_file::moveFile($filename, $thumbDir . '/' . $thumbID . '/' . $file);
			}
		}
		//delete old files
		foreach(glob($thumbDir . '/*_*_*.*', GLOB_NOSORT) as $filename){
			we_base_file::deleteLocalFile($filename);
		}
	}

	private static function movedFiles(): void{
		if(file_exists(WE_INCLUDES_PATH . 'conf/we_conf_language.inc.php')){
			foreach([
			'cliConfig.inc.php',
			'we_active_integrated_modules.inc.php',
			'we_conf_global.inc.php',
			'we_conf.inc.php',
			'we_conf_language.inc.php'
			] as $file){
				if(file_exists(WE_INCLUDES_PATH . 'conf/' . $file)){
					we_base_file::moveFile(WE_INCLUDES_PATH . 'conf/' . $file, WE_CUSTOM_PATH . 'conf/' . $file);
				}
			}
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'liveUpdate/includes/proxysettings.inc.php'))){
			we_base_file::moveFile($oldPath, WE_CUSTOM_PATH . 'proxysettings.inc.php');
		}

		if(file_exists(WEBEDITION_PATH . 'agency.php')){
			we_base_file::moveFile(WEBEDITION_PATH . 'agency.php', WE_CUSTOM_PATH . 'agency.inc.php');
		}
		if(!empty($GLOBALS['weDefaultFrontendLanguage'])){
			we_base_preferences::we_writeLanguageConfig($GLOBALS['weDefaultFrontendLanguage'], $GLOBALS['weFrontendLanguages']);
		}
		if(file_exists(WEBEDITION_PATH . 'we/templates')){
			we_base_file::deleteLocalFolder(TEMPLATES_PATH, true, true);
			we_base_file::moveFile(WEBEDITION_PATH . 'we/templates', TEMPLATES_PATH);
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'we/include/we_tags/custom_tags'))){
			we_base_file::deleteLocalFolder(($newPath = WE_CUSTOM_PATH . we_weTag_tag::DIR), true, true);
			we_base_file::moveFile($oldPath, $newPath);
			we_base_file::deleteLocalFolder(WEBEDITION_PATH . 'we/include/we_tags', true, true);
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'we/include/we_hook/custom_hooks'))){
			we_base_file::deleteLocalFolder(($newPath = WE_CUSTOM_PATH . we_hook_handler::DIR), true, true);
			we_base_file::moveFile($oldPath, $newPath);
			we_base_file::deleteLocalFolder(WEBEDITION_PATH . 'we/include/we_hook', true, true);
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'agency.inc.php'))){
			we_base_file::moveFile($oldPath, WE_CUSTOM_PATH . 'agency.inc.php');
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'maintenance.inc.php'))){
			we_base_file::moveFile($oldPath, WE_CUSTOM_PATH . 'maintenance.inc.php');
		}
		if(file_exists(WEBEDITION_PATH . 'we/versions')){
			we_base_file::deleteLocalFolder(WE_VERSION_PATH, true, true);
			we_base_file::moveFile(WEBEDITION_PATH . 'we/versions', WE_VERSION_PATH);
		}
		if(file_exists(WEBEDITION_PATH . 'we_backup/data')){
			we_base_file::deleteLocalFolder(WE_BACKUP_PATH, true, true);
			we_base_file::moveFile(WEBEDITION_PATH . 'we_backup/data', WE_BACKUP_PATH);
		}
		if(file_exists(WEBEDITION_PATH . 'site')){
			we_base_file::deleteLocalFolder(WE_SITE_DIR, true, true);
			we_base_file::moveFile(WEBEDITION_PATH . 'site', WE_DOCUMENT_ROOT . WE_SITE_DIR);
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'we/include/weTagWizard/we_tags/custom_tags'))){
			we_base_file::deleteLocalFolder(($newPath = WEBEDITION_PATH . 'we/include/old-tw-Tags'), true, true);
			we_base_file::moveFile($oldPath, $newPath);
			we_base_file::deleteLocalFolder(WEBEDITION_PATH . 'we/include/weTagWizard', true, true);
		}
		if(file_exists(($oldPath = WEBEDITION_PATH . 'we/include/weCodeWizard/data/custom'))){
			we_base_file::deleteLocalFolder(($newPath = WE_CUSTOM_PATH . 'codesnippets'), true, true);
			we_base_file::moveFile($oldPath, $newPath);
			we_base_file::deleteLocalFolder(WEBEDITION_PATH . 'we/include/weCodeWizard', true, true);
		}
	}

	private static function updateTableToUTF8(we_database_base $db, string $table, bool $conIsUtf8): array{
		$arr = $db->getTableCreateArray($table);
		$charset = $collate = '';
		$matches = $dates = $changeCols = [];

		foreach($arr as $c){
			'@phan-var string $c';
			if(preg_match('/DEFAULT CHARSET=([^ ,]+) COLLATE=([^ ,]+)/i', $c, $matches)){
				$charset = $matches[1];
				$collate = $matches[2];
				break;
			} elseif(preg_match('/\s*(PRIMARY KEY|UNIQUE KEY|KEY)/i', $c) ||
				preg_match('/ (VIRTUAL|STORED)/i', $c)
			){
				continue;
			} elseif(preg_match('/`([^`]+)`(.*(set|enum|char|text).*),$/i', $c, $matches)){
				//skip mixed cols in utf8
				if(stripos($c, 'COLLATE ' . we_database_base::DB_COLLATION) !== false){
					continue;
				}
				$matches[2] = preg_replace('/COLLATE [^ ]+/i', '', $matches[2]);
				$matches[2] = preg_replace('/CHARACTER SET [^ ]+/i', '', $matches[2]);
				$changeCols[$matches[1]] = $matches[2];
			} elseif(preg_match('/`([^`]+)`\s*((date|timestamp).*),$/i', $c, $matches)){
				$dates[$matches[1]] = stripos($matches[2], 'null') !== false;
			}
		}
		foreach($dates as $col => $isNull){
			$db->query('UPDATE `' . $db->escape($table) . '` SET `' . $col . '`=' . ($isNull ? 'NULL' : '1970-01-01 10:00:00') . ' WHERE YEAR(`' . $col . '`)=0');
		}
		//check old charset
		if($conIsUtf8){//utf8 connection present
			if($charset === we_database_base::DB_CHARSET){//tablecharset matches assume all cols match
				if($collate === we_database_base::DB_COLLATION){
					return ['*' => 'unchanged'];
				}
				$db->query('ALTER TABLE `' . $db->escape($table) . '` CONVERT TO CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION . ', CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION);
				return ['*' => 'converted'];
			}
			//utf-8 data was written to non utf-8 fields, e.g. latin1
			//make all cols binary
			foreach($changeCols as $col => $def){
				$newtyp = strtr(strtolower($def), ['text' => 'blob', 'char' => 'binary']);
				$db->changeColType($table, $col, $newtyp);
			}
			$db->query('ALTER TABLE `' . $db->escape($table) . '` CONVERT TO CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION . ', CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION);
			//reset cols
			foreach($changeCols as $col => $def){
				$db->changeColType($table, $col, $def);
			}
			return $changeCols;
		}
		//non utf8 data was written to utf8 table - data is either incomplete or english, chars <127
		/* if($charset === we_database_base::DB_CHARSET && $collate !== we_database_base::DB_COLLATION){
		  //all ok
		  return ['*'=>'fail1'];
		  } */
		//just convert the table and all cols
		$db->query('ALTER TABLE `' . $db->escape($table) . '` CONVERT TO CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION . ', CHARACTER SET ' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION);
//reset cols
		foreach($changeCols as $col => $def){
			$db->changeColType($table, $col, $def);
		}
		return $changeCols;
	}

	public static function convertToUtf8(we_database_base $db): void{
		if(!defined('DB_CHARSET')){
			//no conversion needed
			return;
		}
		$tables = $db->table_names(TBL_PREFIX === '' ? 'tbl%' : TBL_PREFIX . '%');
		$connection = $db->getCurrentCharset();
		$collation = we_database_base::getCollation(); //@phan-suppress-current-line PhanDeprecatedFunction
		$charset = we_database_base::getCharset(); //@phan-suppress-current-line PhanDeprecatedFunction

		we_base_preferences::loadConfigs();
//conf_conf
		we_base_preferences::$config_files['conf_conf']['content'] = we_base_preferences::changeSourceCode('define', we_base_preferences::$config_files['conf_conf']['content'], 'DB_SET_CHARSET', we_database_base::DB_CHARSET, false);
		$file = we_base_preferences::$config_files['conf_global']['content'];
		$file = we_base_preferences::changeSourceCode('define', $file, 'DB_CHARSET', we_database_base::DB_CHARSET, false);
		$file = we_base_preferences::changeSourceCode('define', $file, 'WE_BACKENDCHARSET', 'utf8', false);
		$file = we_base_preferences::changeSourceCode('define', $file, 'DB_COLLATION', we_database_base::DB_COLLATION, false);
		we_base_preferences::$config_files['conf_global']['content'] = $file;
		we_base_preferences::saveConfigs();

		$ret = [
			'con' => $connection,
			'coll' => $collation,
			'char' => $charset,
			'tables' => []
		];
		$conIsUtf8 = ($connection === we_database_base::DB_CHARSET);
		//try to set default collation on DB
		$db->query('ALTER DATABASE `' . DB_DATABASE . '` DEFAULT CHARSET=' . we_database_base::DB_CHARSET . ' COLLATE ' . we_database_base::DB_COLLATION);
		foreach($tables as $tab){
			$table = $tab['table_name'];
			$val = self::updateTableToUTF8($db, $table, $conIsUtf8);
			if($val){
				$ret['tables'][$table] = [$charset, $val];
			}
		}
		if(!empty($ret['tables'])){
			t_e($ret);
		}
	}

	private static function fixBadValues(we_database_base $db): void{
		$tables = $db->table_names(TBL_PREFIX === '' ? 'tbl%' : TBL_PREFIX . '%');
		$dateCols = [we_database_base::TYPE_TIMESTAMP, we_database_base::TYPE_DATE, we_database_base::TYPE_DATETIME];
		foreach($tables as $tab){
			$table = $tab['table_name'];
			$data = $db->getAllFirstq('SHOW COLUMNS FROM ' . $table . ' WHERE Type LIKE "' . we_database_base::TYPE_ENUM . '%" OR Type IN ("' . implode('","', $dateCols) . '")', true, MYSQLI_ASSOC);
			foreach($data as $col => $d){
				$hasNull = strtoupper($d['Null']) === 'YES';
				switch($d['Type']){
					case we_database_base::TYPE_DATE:
						$db->query('UPDATE ' . $table . ' SET `' . $col . '`=' . ($hasNull ? 'NULL' : '"1970-01-01"') . ' WHERE YEAR(`' . $col . '`)=0');
						break;
					case we_database_base::TYPE_DATETIME:
					case we_database_base::TYPE_TIMESTAMP:
						$db->query('UPDATE ' . $table . ' SET `' . $col . '`=' . ($hasNull ? 'NULL' : '"1970-01-01 10:00:00"') . ' WHERE YEAR(`' . $col . '`)=0');
						break;
					default://enum
						$vals = explode('\',\'', strtr($d['Type'], ['enum(\'' => '', '\')' => '']));
						$db->query('UPDATE ' . $table . ' SET ' . we_database_base::arraySetter([
								$col => ($hasNull ? null : $d['Default'] ?? $vals[0])
							]) . ' WHERE `' . $col . '`+0=0');
				}
			}
		}
	}

	/**
	 *
	 * @param string $what
	 * @param array $progress
	 * @return array|array{what:string,pos:int,max:int,text:string}
	 */
	public static function doUpdate(string $what = '', array $progress = []): array{
		$db = new DB_WE();
		self::meassure('start');
		//
		unset($progress['what'], $progress['text']);
		switch($what){
			case 'internal'://if we are in liveupdate, initial db updates already triggered
				self::replayUpdateDB();
				self::meassure('replayUpdateDB');
			//no break;
			case '':
			default:
				self::movedFiles();
				self::fixInconsistentTables($db);
				self::meassure('fixInconsistentTables');
				self::updateVersionsTable($db);
				$db2 = new DB_WE();
				self::updateVersionsTable2($db, $db2);
				self::meassure('versions');
				self::updateCustomerFilters($db);
				self::meassure('customerFilter');
				self::updateCustomer($db);
				self::meassure('customer');
				self::updateSetting();
				self::meassure('setting');
				self::updateLinkList($db, $db2);
				self::meassure('updateLinkList');
				self::updateScheduler($db, $db2);
				self::meassure('updateScheduler');
				self::updateThumbnails($db);
				self::meassure('updateThumbnails');
				return ['what' => 'object', 'text' => 'Classes (1)'];
			case 'object':
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$ret = self::updateObjectFilesX($db, $progress);
					if($ret){
						return array_merge($ret, ['what' => 'object']);
					}
					return ['what' => 'tempTable', 'text' => 'Temptable'];
				}
			case 'tempTable':
				$ret = self::updateTempTable($db, $progress);
				if($ret){
					return array_merge($ret, ['what' => 'tempTable']);
				}
				self::meassure('updateTemp');
				return ['what' => 'objectX', 'text' => 'Classes (2)'];
			case 'objectX':
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					self::updateObjectFiles2X($db);
				}
				return ['what' => 'dbClean', 'text' => 'DB Cleanup'];
			case 'dbClean':
				self::updateObjects($db);
				self::fixBadValues($db);
				return ['what' => 'utf8', 'text' => 'UTF-8 Conversion'];
			case 'utf8':
				self::convertToUtf8($db);
				return ['what' => 'cleanUpFiles', 'text' => 'Cleanup Files'];
			case 'cleanUpFiles':
				self::removeObsoleteFiles();
				return ['what' => 'replayUpdateDB', 'text' => 'DB-Update'];
			case 'replayUpdateDB':
				self::replayUpdateDB();
				self::fixBadValues($db);
				self::meassure('replayUpdateDB');
				self::meassure('-1');
		}
		return [];
	}

}
