<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_typeAttribute extends we_weTag_attribute{
	/**
	 * @var array
	 */
	private $Options;

	/**
	 * @param string $name
	 * @param we_weTag_option[] $options
	 * @param boolean $required
	 */
	function __construct(string $name, array $options = [], bool $required = true, string $module = '', string $description = '', bool $deprecated = false){
		parent::__construct($name, $required, $module, $description, $deprecated);
		$this->Options = parent::getUseOptions($options);
		foreach($this->Options as &$option){
			$option->addTypeAttribute($this);
		}
		// overwrite value if needed
		if($this->Value === false){
			$this->Value = '-';
		}
	}

	/**
	 * @return string
	 */
	function getCodeForTagWizard(): string{
		$entries = ['' => g_l('taged', '[select_type]')];

		foreach($this->Options as $option){
			$entries[$option->Value] = ($option->getName() === '-' ? '' : $option->getName());
		}

		$js = "we_cmd('switch_type', this.value);";

		$select = (new we_html_select(['name' => $this->Name,
				'id' => $this->getIdName(),
				'onchange' => $js,
				'class' => 'defaultfont selectinput'
				]))
			->addOptions($entries);

		return '
<table class="attribute">
<tr>
	<td class="attributeName">' . $this->getLabelCodeForTagWizard() . '</td>
	<td class="attributeField">' . $select->getHtml() . '</td>
</tr>
</table>';
	}

	/**
	 * @return array
	 */
	function getOptions(): array{
		return $this->Options;
	}

}
